@@ -292,7 +292,8 @@ bool make_user_info_netlogon_interactive(auth_usersupplied_info **user_info,
 	unsigned char local_nt_response[24];
 	unsigned char key[16];
 	
-	memcpy(key, dc_sess_key, 16);
+	ZERO_STRUCT(key);
+	memcpy(key, dc_sess_key, 8);
 	
 	if (lm_interactive_pwd)
 		memcpy(lm_pwd, lm_interactive_pwd, sizeof(lm_pwd));
@@ -507,11 +508,19 @@ static auth_serversupplied_info *make_server_info(TALLOC_CTX *mem_ctx)
 	   which may save us from giving away root access if there
 	   is a bug in allocating these fields. */
 
-	result->uid = -1;
-	result->gid = -1;
+	result->utok.uid = -1;
+	result->utok.gid = -1;
 	return result;
 }
 
+static char *sanitize_username(TALLOC_CTX *mem_ctx, const char *username)
+{
+	fstring tmp;
+
+	alpha_strcpy(tmp, username, ". _-$", sizeof(tmp));
+	return talloc_strdup(mem_ctx, tmp);
+}
+
 /***************************************************************************
  Is the incoming username our own machine account ?
  If so, the connection is almost certainly from winbindd.
@@ -546,6 +555,9 @@ NTSTATUS make_server_info_sam(auth_serversupplied_info **server_info,
 	struct passwd *pwd;
 	gid_t *gids;
 	auth_serversupplied_info *result;
+	int i;
+	size_t num_gids;
+	DOM_SID unix_group_sid;
 	const char *username = pdb_get_username(sampass);
 	NTSTATUS status;
 
@@ -561,14 +573,23 @@ NTSTATUS make_server_info_sam(auth_serversupplied_info **server_info,
 	}
 
 	result->sam_account = sampass;
+	/* Ensure thaat the sampass will be freed with the result */
+	talloc_steal(result, sampass);
 	result->unix_name = pwd->pw_name;
 	/* Ensure that we keep pwd->pw_name, because we will free pwd below */
 	talloc_steal(result, pwd->pw_name);
-	result->gid = pwd->pw_gid;
-	result->uid = pwd->pw_uid;
+	result->utok.gid = pwd->pw_gid;
+	result->utok.uid = pwd->pw_uid;
 
 	TALLOC_FREE(pwd);
 
+	result->sanitized_username = sanitize_username(result,
+						       result->unix_name);
+	if (result->sanitized_username == NULL) {
+		TALLOC_FREE(result);
+		return NT_STATUS_NO_MEMORY;
+	}
+
 	if (IS_DC && is_our_machine_account(username)) {
 		/*
 		 * Ensure for a connection from our own
@@ -610,6 +631,30 @@ NTSTATUS make_server_info_sam(auth_serversupplied_info **server_info,
 		}
 	}
 
+	/* Add the "Unix Group" SID for each gid to catch mapped groups
+	   and their Unix equivalent.  This is to solve the backwards 
+	   compatibility problem of 'valid users = +ntadmin' where 
+	   ntadmin has been paired with "Domain Admins" in the group 
+	   mapping table.  Otherwise smb.conf would need to be changed
+	   to 'valid user = "Domain Admins"'.  --jerry */
+	
+	num_gids = result->num_sids;
+	for ( i=0; i<num_gids; i++ ) {
+		if ( !gid_to_unix_groups_sid( gids[i], &unix_group_sid ) ) {
+			DEBUG(1,("make_server_info_sam: Failed to create SID "
+				"for gid %d!\n", gids[i]));
+			continue;
+		}
+		status = add_sid_to_array_unique(result, &unix_group_sid,
+						 &result->sids,
+						 &result->num_sids);
+		if (!NT_STATUS_IS_OK(status)) {
+			result->sam_account = NULL; /* Don't free on error exit. */
+			TALLOC_FREE(result);
+			return status;
+		}
+	}
+
 	/* For now we throw away the gids and convert via sid_to_gid
 	 * later. This needs fixing, but I'd like to get the code straight and
 	 * simple first. */
@@ -620,45 +665,48 @@ NTSTATUS make_server_info_sam(auth_serversupplied_info **server_info,
 		 pdb_get_username(sampass), result->unix_name));
 
 	*server_info = result;
-	/* Ensure thaat the sampass will be freed with the result */
-	talloc_steal(result, sampass);
 
 	return NT_STATUS_OK;
 }
 
-static NTSTATUS log_nt_token(TALLOC_CTX *tmp_ctx, NT_USER_TOKEN *token)
+static NTSTATUS log_nt_token(NT_USER_TOKEN *token)
 {
+	TALLOC_CTX *frame = talloc_stackframe();
 	char *command;
 	char *group_sidstr;
 	size_t i;
 
 	if ((lp_log_nt_token_command() == NULL) ||
 	    (strlen(lp_log_nt_token_command()) == 0)) {
+		TALLOC_FREE(frame);
 		return NT_STATUS_OK;
 	}
 
-	group_sidstr = talloc_strdup(tmp_ctx, "");
+	group_sidstr = talloc_strdup(frame, "");
 	for (i=1; i<token->num_sids; i++) {
 		group_sidstr = talloc_asprintf(
-			tmp_ctx, "%s %s", group_sidstr,
-			sid_string_talloc(tmp_ctx, &token->user_sids[i]));
+			frame, "%s %s", group_sidstr,
+			sid_string_talloc(frame, &token->user_sids[i]));
 	}
 
 	command = talloc_string_sub(
-		tmp_ctx, lp_log_nt_token_command(),
-		"%s", sid_string_talloc(tmp_ctx, &token->user_sids[0]));
-	command = talloc_string_sub(tmp_ctx, command, "%t", group_sidstr);
+		frame, lp_log_nt_token_command(),
+		"%s", sid_string_talloc(frame, &token->user_sids[0]));
+	command = talloc_string_sub(frame, command, "%t", group_sidstr);
 
 	if (command == NULL) {
+		TALLOC_FREE(frame);
 		return NT_STATUS_NO_MEMORY;
 	}
 
 	DEBUG(8, ("running command: [%s]\n", command));
 	if (smbrun(command, NULL) != 0) {
 		DEBUG(0, ("Could not log NT token\n"));
+		TALLOC_FREE(frame);
 		return NT_STATUS_ACCESS_DENIED;
 	}
 
+	TALLOC_FREE(frame);
 	return NT_STATUS_OK;
 }
 
@@ -669,10 +717,8 @@ static NTSTATUS log_nt_token(TALLOC_CTX *tmp_ctx, NT_USER_TOKEN *token)
 
 NTSTATUS create_local_token(auth_serversupplied_info *server_info)
 {
-	TALLOC_CTX *mem_ctx;
 	NTSTATUS status;
 	size_t i;
-	struct dom_sid tmp_sid;
 
 	/*
 	 * If winbind is not around, we can not make much use of the SIDs the
@@ -681,15 +727,15 @@ NTSTATUS create_local_token(auth_serversupplied_info *server_info)
 	 */
 
 	if (((lp_server_role() == ROLE_DOMAIN_MEMBER) && !winbind_ping()) ||
-	    (server_info->was_mapped)) {
+	    (server_info->nss_token)) {
 		status = create_token_from_username(server_info,
 						    server_info->unix_name,
 						    server_info->guest,
-						    &server_info->uid,
-						    &server_info->gid,
+						    &server_info->utok.uid,
+						    &server_info->utok.gid,
 						    &server_info->unix_name,
 						    &server_info->ptok);
-		
+
 	} else {
 		server_info->ptok = create_local_nt_token(
 			server_info,
@@ -703,11 +749,11 @@ NTSTATUS create_local_token(auth_serversupplied_info *server_info)
 	if (!NT_STATUS_IS_OK(status)) {
 		return status;
 	}
-	
+
 	/* Convert the SIDs to gids. */
 
-	server_info->n_groups = 0;
-	server_info->groups = NULL;
+	server_info->utok.ngroups = 0;
+	server_info->utok.groups = NULL;
 
 	/* Start at index 1, where the groups start. */
 
@@ -720,58 +766,19 @@ NTSTATUS create_local_token(auth_serversupplied_info *server_info)
 				   "ignoring it\n", sid_string_dbg(sid)));
 			continue;
 		}
-		add_gid_to_array_unique(server_info, gid, &server_info->groups,
-					&server_info->n_groups);
-	}
-
-	if (!uid_to_unix_users_sid(server_info->uid, &tmp_sid)) {
-		DEBUG(1,("create_local_token: Failed to create SID "
-			"for uid %d!\n", server_info->uid));
-	}
-	add_sid_to_array_unique(server_info->ptok, &tmp_sid,
-				&server_info->ptok->user_sids,
-				&server_info->ptok->num_sids);
-
-	if (!gid_to_unix_groups_sid( server_info->gid, &tmp_sid)) {
-		DEBUG(1,("create_local_token: Failed to create SID "
-			"for gid %d!\n", server_info->gid));
-	}
-	add_sid_to_array_unique(server_info->ptok, &tmp_sid,
-				&server_info->ptok->user_sids,
-				&server_info->ptok->num_sids);
-
-	for ( i=0; i<server_info->n_groups; i++ ) {
-		if (!gid_to_unix_groups_sid( server_info->groups[i], &tmp_sid ) ) {
-			DEBUG(1,("create_local_token: Failed to create SID "
-				"for gid %d!\n", server_info->groups[i]));
-			continue;
-		}
-		add_sid_to_array_unique(server_info->ptok, &tmp_sid,
-					&server_info->ptok->user_sids,
-					&server_info->ptok->num_sids);
+		add_gid_to_array_unique(server_info, gid,
+					&server_info->utok.groups,
+					&server_info->utok.ngroups);
 	}
 
 	debug_nt_user_token(DBGC_AUTH, 10, server_info->ptok);
-	debug_unix_user_token(DBGC_AUTH, 10,
-			      server_info->uid,
-			      server_info->gid,
-			      server_info->n_groups,
-			      server_info->groups);
 
-	mem_ctx = talloc_new(NULL);
-	if (mem_ctx == NULL) {
-		DEBUG(0, ("talloc_new failed\n"));
-		return NT_STATUS_NO_MEMORY;
-	}
-
-	status = log_nt_token(mem_ctx, server_info->ptok);
-
-	TALLOC_FREE(mem_ctx);
+	status = log_nt_token(server_info->ptok);
 	return status;
 }
 
 /*
- * Create an artificial NT token given just a username. (Initially intended
+ * Create an artificial NT token given just a username. (Initially indended
  * for force user)
  *
  * We go through lookup_name() to avoid problems we had with 'winbind use
@@ -824,6 +831,12 @@ NTSTATUS create_token_from_username(TALLOC_CTX *mem_ctx, const char *username,
 		goto done;
 	}
 
+	if (!sid_to_uid(&user_sid, uid)) {
+		DEBUG(1, ("sid_to_uid for %s (%s) failed\n",
+			  username, sid_string_dbg(&user_sid)));
+		goto done;
+	}
+
 	if (sid_check_is_in_our_domain(&user_sid)) {
 		bool ret;
 
@@ -866,33 +879,6 @@ NTSTATUS create_token_from_username(TALLOC_CTX *mem_ctx, const char *username,
 		*found_username = talloc_strdup(mem_ctx,
 						pdb_get_username(sam_acct));
 
-		/*
-		 * If the SID from lookup_name() was the guest sid, passdb knows
-		 * about the mapping of guest sid to lp_guestaccount()
-		 * username and will return the unix_pw info for a guest
-		 * user. Use it if it's there, else lookup the *uid details
-		 * using getpwnam_alloc(). See bug #6291 for details. JRA.
-		 */
-
-		/* We must always assign the *uid. */
-		if (sam_acct->unix_pw == NULL) {
-			struct passwd *pwd = getpwnam_alloc(sam_acct, *found_username );
-			if (!pwd) {
-				DEBUG(10, ("getpwnam_alloc failed for %s\n",
-					*found_username));
-				result = NT_STATUS_NO_SUCH_USER;
-				goto done;
-			}
-			result = samu_set_unix(sam_acct, pwd );
-			if (!NT_STATUS_IS_OK(result)) {
-				DEBUG(10, ("samu_set_unix failed for %s\n",
-					*found_username));
-				result = NT_STATUS_NO_SUCH_USER;
-				goto done;
-			}
-		}
-		*uid = sam_acct->unix_pw->pw_uid;
-
 	} else 	if (sid_check_is_in_unix_users(&user_sid)) {
 
 		/* This is a unix user not in passdb. We need to ask nss
@@ -908,13 +894,6 @@ NTSTATUS create_token_from_username(TALLOC_CTX *mem_ctx, const char *username,
 
 	unix_user:
 
-		if (!sid_to_uid(&user_sid, uid)) {
-			DEBUG(1, ("unix_user case, sid_to_uid for %s (%s) failed\n",
-				  username, sid_string_dbg(&user_sid)));
-			result = NT_STATUS_NO_SUCH_USER;
-			goto done;
-		}
-
 		uid_to_unix_users_sid(*uid, &user_sid);
 
 		pass = getpwuid_alloc(tmp_ctx, *uid);
@@ -964,14 +943,6 @@ NTSTATUS create_token_from_username(TALLOC_CTX *mem_ctx, const char *username,
 
 		uint32 dummy;
 
-		/* We must always assign the *uid. */
-		if (!sid_to_uid(&user_sid, uid)) {
-			DEBUG(1, ("winbindd case, sid_to_uid for %s (%s) failed\n",
-				  username, sid_string_dbg(&user_sid)));
-			result = NT_STATUS_NO_SUCH_USER;
-			goto done;
-		}
-
 		num_group_sids = 1;
 		group_sids = TALLOC_ARRAY(tmp_ctx, DOM_SID, num_group_sids);
 		if (group_sids == NULL) {
@@ -1103,9 +1074,8 @@ bool user_in_group(const char *username, const char *groupname)
 	return user_in_group_sid(username, &group_sid);
 }
 
-
 /***************************************************************************
- Make (and fill) a user_info struct from a 'struct passwd' by conversion 
+ Make (and fill) a server_info struct from a 'struct passwd' by conversion
  to a struct samu
 ***************************************************************************/
 
@@ -1138,9 +1108,19 @@ NTSTATUS make_server_info_pw(auth_serversupplied_info **server_info,
 	}
 
 	result->sam_account = sampass;
+
 	result->unix_name = talloc_strdup(result, unix_username);
-	result->uid = pwd->pw_uid;
-	result->gid = pwd->pw_gid;
+	result->sanitized_username = sanitize_username(result, unix_username);
+
+	if ((result->unix_name == NULL)
+	    || (result->sanitized_username == NULL)) {
+		TALLOC_FREE(sampass);
+		TALLOC_FREE(result);
+		return NT_STATUS_NO_MEMORY;
+	}
+
+	result->utok.uid = pwd->pw_uid;
+	result->utok.gid = pwd->pw_gid;
 
 	status = pdb_enum_group_memberships(result, sampass,
 					    &result->sids, &gids,
@@ -1227,6 +1207,7 @@ static NTSTATUS make_new_server_info_guest(auth_serversupplied_info **server_inf
 	DOM_SID guest_sid;
 	bool ret;
 	char zeros[16];
+	fstring tmp;
 
 	if ( !(sampass = samu_new( NULL )) ) {
 		return NT_STATUS_NO_MEMORY;
@@ -1265,27 +1246,73 @@ static NTSTATUS make_new_server_info_guest(auth_serversupplied_info **server_inf
 	(*server_info)->user_session_key = data_blob(zeros, sizeof(zeros));
 	(*server_info)->lm_session_key = data_blob(zeros, sizeof(zeros));
 
+	alpha_strcpy(tmp, pdb_get_username(sampass), ". _-$", sizeof(tmp));
+	(*server_info)->sanitized_username = talloc_strdup(*server_info, tmp);
+
 	return NT_STATUS_OK;
 }
 
-static auth_serversupplied_info *copy_serverinfo(auth_serversupplied_info *src)
+/****************************************************************************
+  Fake a auth_serversupplied_info just from a username
+****************************************************************************/
+
+NTSTATUS make_serverinfo_from_username(TALLOC_CTX *mem_ctx,
+				       const char *username,
+				       bool is_guest,
+				       struct auth_serversupplied_info **presult)
+{
+	struct auth_serversupplied_info *result;
+	struct passwd *pwd;
+	NTSTATUS status;
+
+	pwd = getpwnam_alloc(talloc_tos(), username);
+	if (pwd == NULL) {
+		return NT_STATUS_NO_SUCH_USER;
+	}
+
+	status = make_server_info_pw(&result, pwd->pw_name, pwd);
+
+	TALLOC_FREE(pwd);
+
+	if (!NT_STATUS_IS_OK(status)) {
+		return status;
+	}
+
+	result->nss_token = true;
+	result->guest = is_guest;
+
+	status = create_local_token(result);
+
+	if (!NT_STATUS_IS_OK(status)) {
+		TALLOC_FREE(result);
+		return status;
+	}
+
+	*presult = result;
+	return NT_STATUS_OK;
+}
+
+
+struct auth_serversupplied_info *copy_serverinfo(TALLOC_CTX *mem_ctx,
+						 const auth_serversupplied_info *src)
 {
 	auth_serversupplied_info *dst;
 
-	dst = make_server_info(NULL);
+	dst = make_server_info(mem_ctx);
 	if (dst == NULL) {
 		return NULL;
 	}
 
 	dst->guest = src->guest;
-	dst->uid = src->uid;
-	dst->gid = src->gid;
-	dst->n_groups = src->n_groups;
-	if (src->n_groups != 0) {
-		dst->groups = (gid_t *)TALLOC_MEMDUP(
-			dst, src->groups, sizeof(gid_t)*dst->n_groups);
+	dst->utok.uid = src->utok.uid;
+	dst->utok.gid = src->utok.gid;
+	dst->utok.ngroups = src->utok.ngroups;
+	if (src->utok.ngroups != 0) {
+		dst->utok.groups = (gid_t *)TALLOC_MEMDUP(
+			dst, src->utok.groups,
+			sizeof(gid_t)*dst->utok.ngroups);
 	} else {
-		dst->groups = NULL;
+		dst->utok.groups = NULL;
 	}
 
 	if (src->ptok) {
@@ -1320,9 +1347,31 @@ static auth_serversupplied_info *copy_serverinfo(auth_serversupplied_info *src)
 		return NULL;
 	}
 
+	dst->sanitized_username = talloc_strdup(dst, src->sanitized_username);
+	if (!dst->sanitized_username) {
+		TALLOC_FREE(dst);
+		return NULL;
+	}
+
 	return dst;
 }
 
+/*
+ * Set a new session key. Used in the rpc server where we have to override the
+ * SMB level session key with SystemLibraryDTC
+ */
+
+bool server_info_set_session_key(struct auth_serversupplied_info *info,
+				 DATA_BLOB session_key)
+{
+	TALLOC_FREE(info->user_session_key.data);
+
+	info->user_session_key = data_blob_talloc(
+		info, session_key.data, session_key.length);
+
+	return (info->user_session_key.data != NULL);
+}
+
 static auth_serversupplied_info *guest_info = NULL;
 
 bool init_guest_info(void)
@@ -1333,9 +1382,10 @@ bool init_guest_info(void)
 	return NT_STATUS_IS_OK(make_new_server_info_guest(&guest_info));
 }
 
-NTSTATUS make_server_info_guest(auth_serversupplied_info **server_info)
+NTSTATUS make_server_info_guest(TALLOC_CTX *mem_ctx,
+				auth_serversupplied_info **server_info)
 {
-	*server_info = copy_serverinfo(guest_info);
+	*server_info = copy_serverinfo(mem_ctx, guest_info);
 	return (*server_info != NULL) ? NT_STATUS_OK : NT_STATUS_NO_MEMORY;
 }
 
@@ -1366,39 +1416,6 @@ bool copy_current_user(struct current_user *dst, struct current_user *src)
 	return True;
 }
 
-bool set_current_user_guest(struct current_user *dst)
-{
-	gid_t *groups;
-	NT_USER_TOKEN *nt_token;
-
-	groups = (gid_t *)memdup(guest_info->groups,
-				 sizeof(gid_t) * guest_info->n_groups);
-	if (groups == NULL) {
-		return False;
-	}
-
-	nt_token = dup_nt_token(NULL, guest_info->ptok);
-	if (nt_token == NULL) {
-		SAFE_FREE(groups);
-		return False;
-	}
-
-	TALLOC_FREE(dst->nt_user_token);
-	SAFE_FREE(dst->ut.groups);
-
-	/* dst->conn is never really dereferenced, it's only tested for
-	 * equality in uid.c */
-	dst->conn = NULL;
-
-	dst->vuid = UID_FIELD_INVALID;
-	dst->ut.uid = guest_info->uid;
-	dst->ut.gid = guest_info->gid;
-	dst->ut.ngroups = guest_info->n_groups;
-	dst->ut.groups = groups;
-	dst->nt_user_token = nt_token;
-	return True;
-}
-
 /***************************************************************************
  Purely internal function for make_server_info_info3
  Fill the sam account from getpwnam
@@ -1619,7 +1636,7 @@ NTSTATUS make_server_info_info3(TALLOC_CTX *mem_ctx,
 	if (!NT_STATUS_IS_OK(nt_status)) {
 		TALLOC_FREE( sam_account );
 		if ( lp_map_to_guest() == MAP_TO_GUEST_ON_BAD_UID ) {
-		 	make_server_info_guest(server_info); 
+			make_server_info_guest(NULL, server_info);
 			return NT_STATUS_OK;
 		}
 		return nt_status;
@@ -1727,10 +1744,17 @@ NTSTATUS make_server_info_info3(TALLOC_CTX *mem_ctx,
 	result->sam_account = sam_account;
 	result->unix_name = talloc_strdup(result, found_username);
 
+	result->sanitized_username = sanitize_username(result,
+						       result->unix_name);
+	if (result->sanitized_username == NULL) {
+		TALLOC_FREE(result);
+		return NT_STATUS_NO_MEMORY;
+	}
+
 	/* Fill in the unix info we found on the way */
 
-	result->uid = uid;
-	result->gid = gid;
+	result->utok.uid = uid;
+	result->utok.gid = gid;
 
 	/* Create a 'combined' list of all SIDs we might want in the SD */
 
@@ -1772,7 +1796,7 @@ NTSTATUS make_server_info_info3(TALLOC_CTX *mem_ctx,
 			sizeof(info3->base.LMSessKey.key));
 	}
 
-	result->was_mapped = username_was_mapped;
+	result->nss_token |= username_was_mapped;
 
 	*server_info = result;
 
@@ -1874,7 +1898,7 @@ NTSTATUS make_server_info_wbcAuthUserInfo(TALLOC_CTX *mem_ctx,
 	if (!NT_STATUS_IS_OK(nt_status)) {
 		TALLOC_FREE( result );
 		if ( lp_map_to_guest() == MAP_TO_GUEST_ON_BAD_UID ) {
-			make_server_info_guest(server_info);
+			make_server_info_guest(NULL, server_info);
 			return NT_STATUS_OK;
 		}
 		return nt_status;
@@ -1965,12 +1989,21 @@ NTSTATUS make_server_info_wbcAuthUserInfo(TALLOC_CTX *mem_ctx,
 	result->sam_account = sam_account;
 	result->unix_name = talloc_strdup(result, found_username);
 
+	result->sanitized_username = sanitize_username(result,
+						       result->unix_name);
 	result->login_server = talloc_strdup(result, info->logon_server);
 
+	if ((result->unix_name == NULL)
+	    || (result->sanitized_username == NULL)
+	    || (result->login_server == NULL)) {
+		TALLOC_FREE(result);
+		return NT_STATUS_NO_MEMORY;
+	}
+
 	/* Fill in the unix info we found on the way */
 
-	result->uid = uid;
-	result->gid = gid;
+	result->utok.uid = uid;
+	result->utok.gid = gid;
 
 	/* Create a 'combined' list of all SIDs we might want in the SD */
 
@@ -2008,7 +2041,7 @@ NTSTATUS make_server_info_wbcAuthUserInfo(TALLOC_CTX *mem_ctx,
 			sizeof(info->lm_session_key));
 	}
 
-	result->was_mapped = username_was_mapped;
+	result->nss_token |= username_was_mapped;
 
 	*server_info = result;
 
