@@ -298,6 +298,7 @@ ATTRIB_MAP_ENTRY sidmap_attr_list[] = {
 {
 	char **values;
 	char *result;
+	size_t converted_size;
 
 	if (attribute == NULL) {
 		return NULL;
@@ -317,7 +318,7 @@ ATTRIB_MAP_ENTRY sidmap_attr_list[] = {
 		return NULL;
 	}
 
-	if (pull_utf8_talloc(mem_ctx, &result, values[0]) == (size_t)-1) {
+	if (!pull_utf8_talloc(mem_ctx, &result, values[0], &converted_size)) {
 		DEBUG(10, ("pull_utf8_talloc failed\n"));
 		ldap_value_free(values);
 		return NULL;
@@ -430,6 +431,7 @@ ATTRIB_MAP_ENTRY sidmap_attr_list[] = {
 
 	if (value != NULL) {
 		char *utf8_value = NULL;
+		size_t converted_size;
 
 		j = 0;
 		if (mods[i]->mod_values != NULL) {
@@ -442,7 +444,7 @@ ATTRIB_MAP_ENTRY sidmap_attr_list[] = {
 			/* notreached. */
 		}
 
-		if (push_utf8_allocate(&utf8_value, value) == (size_t)-1) {
+		if (!push_utf8_allocate(&utf8_value, value, &converted_size)) {
 			smb_panic("smbldap_set_mod: String conversion failure!");
 			/* notreached. */
 		}
@@ -1012,7 +1014,7 @@ static int smbldap_connect_system(struct smbldap_state *ldap_state, LDAP * ldap_
 
 static void smbldap_idle_fn(struct event_context *event_ctx,
 			    struct timed_event *te,
-			    const struct timeval *now,
+			    struct timeval now,
 			    void *private_data);
 
 /**********************************************************************
@@ -1023,13 +1025,6 @@ static int smbldap_open(struct smbldap_state *ldap_state)
 	int rc, opt_rc;
 	bool reopen = False;
 	SMB_ASSERT(ldap_state);
-		
-#ifndef NO_LDAP_SECURITY
-	if (geteuid() != 0) {
-		DEBUG(0, ("smbldap_open: cannot access LDAP when not root\n"));
-		return  LDAP_INSUFFICIENT_ACCESS;
-	}
-#endif
 
 	if ((ldap_state->ldap_struct != NULL) && ((ldap_state->last_ping + SMBLDAP_DONT_PING_TIME) < time(NULL))) {
 
@@ -1084,7 +1079,7 @@ static int smbldap_open(struct smbldap_state *ldap_state)
 		ldap_state->idle_event = event_add_timed(
 			ldap_state->event_context, NULL,
 			timeval_current_ofs(SMBLDAP_IDLE_TIME, 0),
-			"smbldap_idle_fn", smbldap_idle_fn, ldap_state);
+			smbldap_idle_fn, ldap_state);
 	}
 
 	DEBUG(4,("The LDAP server is successfully connected\n"));
@@ -1200,6 +1195,7 @@ static int smbldap_search_ext(struct smbldap_state *ldap_state,
 	char           *utf8_filter;
 	time_t		endtime = time(NULL)+lp_ldap_timeout();
 	struct		timeval timeout;
+	size_t		converted_size;
 
 	SMB_ASSERT(ldap_state);
 	
@@ -1230,7 +1226,7 @@ static int smbldap_search_ext(struct smbldap_state *ldap_state,
 		ZERO_STRUCT(ldap_state->last_rebind);
 	}
 
-	if (push_utf8_allocate(&utf8_filter, filter) == (size_t)-1) {
+	if (!push_utf8_allocate(&utf8_filter, filter, &converted_size)) {
 		return LDAP_NO_MEMORY;
 	}
 
@@ -1396,12 +1392,13 @@ int smbldap_modify(struct smbldap_state *ldap_state, const char *dn, LDAPMod *at
 	int 		attempts = 0;
 	char           *utf8_dn;
 	time_t		endtime = time(NULL)+lp_ldap_timeout();
+	size_t		converted_size;
 
 	SMB_ASSERT(ldap_state);
 
 	DEBUG(5,("smbldap_modify: dn => [%s]\n", dn ));
 
-	if (push_utf8_allocate(&utf8_dn, dn) == (size_t)-1) {
+	if (!push_utf8_allocate(&utf8_dn, dn, &converted_size)) {
 		return LDAP_NO_MEMORY;
 	}
 
@@ -1439,12 +1436,13 @@ int smbldap_add(struct smbldap_state *ldap_state, const char *dn, LDAPMod *attrs
 	int 		attempts = 0;
 	char           *utf8_dn;
 	time_t		endtime = time(NULL)+lp_ldap_timeout();
+	size_t		converted_size;
 	
 	SMB_ASSERT(ldap_state);
 
 	DEBUG(5,("smbldap_add: dn => [%s]\n", dn ));
 
-	if (push_utf8_allocate(&utf8_dn, dn) == (size_t)-1) {
+	if (!push_utf8_allocate(&utf8_dn, dn, &converted_size)) {
 		return LDAP_NO_MEMORY;
 	}
 
@@ -1482,12 +1480,13 @@ int smbldap_delete(struct smbldap_state *ldap_state, const char *dn)
 	int 		attempts = 0;
 	char           *utf8_dn;
 	time_t		endtime = time(NULL)+lp_ldap_timeout();
+	size_t		converted_size;
 	
 	SMB_ASSERT(ldap_state);
 
 	DEBUG(5,("smbldap_delete: dn => [%s]\n", dn ));
 
-	if (push_utf8_allocate(&utf8_dn, dn) == (size_t)-1) {
+	if (!push_utf8_allocate(&utf8_dn, dn, &converted_size)) {
 		return LDAP_NO_MEMORY;
 	}
 
@@ -1573,7 +1572,7 @@ int smbldap_search_suffix (struct smbldap_state *ldap_state,
 
 static void smbldap_idle_fn(struct event_context *event_ctx,
 			    struct timed_event *te,
-			    const struct timeval *now,
+			    struct timeval now,
 			    void *private_data)
 {
 	struct smbldap_state *state = (struct smbldap_state *)private_data;
@@ -1585,13 +1584,13 @@ static void smbldap_idle_fn(struct event_context *event_ctx,
 		return;
 	}
 		
-	if ((state->last_use+SMBLDAP_IDLE_TIME) > now->tv_sec) {
+	if ((state->last_use+SMBLDAP_IDLE_TIME) > now.tv_sec) {
 		DEBUG(10,("ldap connection not idle...\n"));
 
 		state->idle_event = event_add_timed(
 			event_ctx, NULL,
-			timeval_add(now, SMBLDAP_IDLE_TIME, 0),
-			"smbldap_idle_fn", smbldap_idle_fn,
+			timeval_add(&now, SMBLDAP_IDLE_TIME, 0),
+			smbldap_idle_fn,
 			private_data);
 		return;
 	}
@@ -1654,14 +1653,16 @@ NTSTATUS smbldap_init(TALLOC_CTX *mem_ctx, struct event_context *event_ctx,
 char *smbldap_get_dn(LDAP *ld, LDAPMessage *entry)
 {
 	char *utf8_dn, *unix_dn;
+	size_t converted_size;
 
 	utf8_dn = ldap_get_dn(ld, entry);
 	if (!utf8_dn) {
 		DEBUG (5, ("smbldap_get_dn: ldap_get_dn failed\n"));
 		return NULL;
 	}
-	if (pull_utf8_allocate(&unix_dn, utf8_dn) == (size_t)-1) {
-		DEBUG (0, ("smbldap_get_dn: String conversion failure utf8 [%s]\n", utf8_dn));
+	if (!pull_utf8_allocate(&unix_dn, utf8_dn, &converted_size)) {
+		DEBUG (0, ("smbldap_get_dn: String conversion failure utf8 "
+			   "[%s]\n", utf8_dn));
 		return NULL;
 	}
 	ldap_memfree(utf8_dn);
@@ -1672,13 +1673,14 @@ char *smbldap_get_dn(LDAP *ld, LDAPMessage *entry)
 			       LDAPMessage *entry)
 {
 	char *utf8_dn, *unix_dn;
+	size_t converted_size;
 
 	utf8_dn = ldap_get_dn(ld, entry);
 	if (!utf8_dn) {
 		DEBUG (5, ("smbldap_get_dn: ldap_get_dn failed\n"));
 		return NULL;
 	}
-	if (pull_utf8_talloc(mem_ctx, &unix_dn, utf8_dn) == (size_t)-1) {
+	if (!pull_utf8_talloc(mem_ctx, &unix_dn, utf8_dn, &converted_size)) {
 		DEBUG (0, ("smbldap_get_dn: String conversion failure utf8 "
 			   "[%s]\n", utf8_dn));
 		return NULL;
