@@ -21,7 +21,7 @@
 
 
 #include "includes.h"
-#include "librpc/gen_ndr/security.h"
+#include "librpc/gen_ndr/ndr_security.h"
 
 /*
   return the wire size of a dom_sid
