@@ -273,8 +273,9 @@ net_share_enum_rpc(struct cli_state *cli,
 	uint32_t total_entries = 0;
 
         /* Open the server service pipe */
-        pipe_hnd = cli_rpc_pipe_open_noauth(cli, PI_SRVSVC, &nt_status);
-        if (!pipe_hnd) {
+        nt_status = cli_rpc_pipe_open_noauth(cli, &ndr_table_srvsvc.syntax_id,
+					     &pipe_hnd);
+        if (!NT_STATUS_IS_OK(nt_status)) {
                 DEBUG(1, ("net_share_enum_rpc pipe open fail!\n"));
                 return -1;
         }
@@ -287,7 +288,7 @@ net_share_enum_rpc(struct cli_state *cli,
 
         /* Issue the NetShareEnum RPC call and retrieve the response */
 	nt_status = rpccli_srvsvc_NetShareEnumAll(pipe_hnd, talloc_tos(),
-						  pipe_hnd->cli->desthost,
+						  pipe_hnd->desthost,
 						  &info_ctr,
 						  preferred_len,
 						  &total_entries,
@@ -319,7 +320,7 @@ net_share_enum_rpc(struct cli_state *cli,
 
 done:
         /* Close the server service pipe */
-        cli_rpc_pipe_close(pipe_hnd);
+        TALLOC_FREE(pipe_hnd);
 
         /* Tell 'em if it worked */
         return W_ERROR_IS_OK(result) ? 0 : -1;
@@ -1168,8 +1169,7 @@ SMBC_mkdir_ctx(SMBCCTX *context,
 	if (!cli_resolve_path(frame, "", srv->cli, path,
                               &targetcli, &targetpath)) {
 		d_printf("Could not resolve %s\n", path);
-                errno = ENOENT;
-                TALLOC_FREE(frame);
+		TALLOC_FREE(frame);
 		return -1;
 	}
 	/*d_printf(">>>mkdir: resolved path as %s\n", targetpath);*/
@@ -1276,7 +1276,6 @@ SMBC_rmdir_ctx(SMBCCTX *context,
 	if (!cli_resolve_path(frame, "", srv->cli, path,
                               &targetcli, &targetpath)) {
 		d_printf("Could not resolve %s\n", path);
-                errno = ENOENT;
 		TALLOC_FREE(frame);
 		return -1;
 	}
@@ -1740,8 +1739,6 @@ SMBC_unlink_ctx(SMBCCTX *context,
 	if (!cli_resolve_path(frame, "", srv->cli, path,
                               &targetcli, &targetpath)) {
 		d_printf("Could not resolve %s\n", path);
-                errno = ENOENT;
-                errno = ENOENT;
 		TALLOC_FREE(frame);
 		return -1;
 	}
@@ -1908,7 +1905,6 @@ SMBC_rename_ctx(SMBCCTX *ocontext,
 	if (!cli_resolve_path(frame, "", srv->cli, path1,
                               &targetcli1, &targetpath1)) {
 		d_printf("Could not resolve %s\n", path1);
-                errno = ENOENT;
 		TALLOC_FREE(frame);
 		return -1;
 	}
@@ -1917,7 +1913,6 @@ SMBC_rename_ctx(SMBCCTX *ocontext,
 	if (!cli_resolve_path(frame, "", srv->cli, path2,
                               &targetcli2, &targetpath2)) {
 		d_printf("Could not resolve %s\n", path2);
-                errno = ENOENT;
 		TALLOC_FREE(frame);
 		return -1;
 	}
