@@ -130,7 +130,7 @@ NTSTATUS make_pdb_method_name(struct pdb_methods **methods, const char *selected
 	trim_char(module_name, ' ', ' ');
 
 
-	DEBUG(5,("Attempting to find an passdb backend to match %s (%s)\n", selected, module_name));
+	DEBUG(5,("Attempting to find a passdb backend to match %s (%s)\n", selected, module_name));
 
 	entry = pdb_find_backend_entry(module_name);
 	
@@ -179,20 +179,26 @@ static struct pdb_methods *pdb_get_methods_reload( bool reload )
 		pdb->free_private_data( &(pdb->private_data) );
 		if ( !NT_STATUS_IS_OK( make_pdb_method_name( &pdb, lp_passdb_backend() ) ) ) {
 			char *msg = NULL;
-			asprintf(&msg, "pdb_get_methods_reload: "
-				"failed to get pdb methods for backend %s\n",
-				lp_passdb_backend());
-			smb_panic(msg);
+			if (asprintf(&msg, "pdb_get_methods_reload: "
+					"failed to get pdb methods for backend %s\n",
+					lp_passdb_backend()) > 0) {
+				smb_panic(msg);
+			} else {
+				smb_panic("pdb_get_methods_reload");
+			}
 		}
 	}
 
 	if ( !pdb ) {
 		if ( !NT_STATUS_IS_OK( make_pdb_method_name( &pdb, lp_passdb_backend() ) ) ) {
 			char *msg = NULL;
-			asprintf(&msg, "pdb_get_methods_reload: "
-				"failed to get pdb methods for backend %s\n",
-				lp_passdb_backend());
-			smb_panic(msg);
+			if (asprintf(&msg, "pdb_get_methods_reload: "
+					"failed to get pdb methods for backend %s\n",
+					lp_passdb_backend()) > 0) {
+				smb_panic(msg);
+			} else {
+				smb_panic("pdb_get_methods_reload");
+			}
 		}
 	}
 
@@ -242,7 +248,7 @@ bool guest_user_info( struct samu *user )
 	NTSTATUS result;
 	const char *guestname = lp_guestaccount();
 	
-	if ( !(pwd = getpwnam_alloc( NULL, guestname ) ) ) {
+	if ( !(pwd = getpwnam_alloc(talloc_autofree_context(), guestname ) ) ) {
 		DEBUG(0,("guest_user_info: Unable to locate guest account [%s]!\n", 
 			guestname));
 		return False;
@@ -599,6 +605,9 @@ static NTSTATUS pdb_default_delete_dom_group(struct pdb_methods *methods,
 	struct group *grp;
 	const char *grp_name;
 
+	/* coverity */
+	map.gid = (gid_t) -1;
+
 	sid_compose(&group_sid, get_global_sam_sid(), rid);
 
 	if (!get_domain_group_from_sid(group_sid, &map)) {
@@ -774,6 +783,9 @@ static NTSTATUS pdb_default_add_groupmem(struct pdb_methods *methods,
 	const char *group_name;
 	uid_t uid;
 
+	/* coverity */
+	map.gid = (gid_t) -1;
+
 	sid_compose(&group_sid, get_global_sam_sid(), group_rid);
 	sid_compose(&member_sid, get_global_sam_sid(), member_rid);
 
@@ -1062,8 +1074,8 @@ bool pdb_new_rid(uint32 *rid)
 		DEBUG(0, ("'algorithmic rid base' is set but a passdb backend "
 			  "without algorithmic RIDs is chosen.\n"));
 		DEBUGADD(0, ("Please map all used groups using 'net groupmap "
-			     "add', set the maximum used RID\n"));
-		DEBUGADD(0, ("and remove the parameter\n"));
+			     "add', set the maximum used RID using\n"));
+		DEBUGADD(0, ("'net setmaxrid' and remove the parameter\n"));
 		return False;
 	}
 
@@ -2016,7 +2028,7 @@ NTSTATUS make_pdb_method( struct pdb_methods **methods )
 {
 	/* allocate memory for the structure as its own talloc CTX */
 
-	if ( !(*methods = TALLOC_ZERO_P(NULL, struct pdb_methods) ) ) {
+	if ( !(*methods = TALLOC_ZERO_P(talloc_autofree_context(), struct pdb_methods) ) ) {
 		return NT_STATUS_NO_MEMORY;
 	}
 
