@@ -29,6 +29,17 @@
 #include <cups/cups.h>
 #include <cups/language.h>
 
+static SIG_ATOMIC_T gotalarm;
+
+/***************************************************************
+ Signal function to tell us we timed out.
+****************************************************************/
+
+static void gotalarm_sig(void)
+{
+        gotalarm = 1;
+}
+
 extern userdom_struct current_user_info;
 
 /*
@@ -50,9 +61,11 @@ static http_t *cups_connect(TALLOC_CTX *frame)
 	http_t *http = NULL;
 	char *server = NULL, *p = NULL;
 	int port;
+	int timeout = lp_cups_connection_timeout();
+	size_t size;
 
 	if (lp_cups_server() != NULL && strlen(lp_cups_server()) > 0) {
-		if (push_utf8_talloc(frame, &server, lp_cups_server()) == (size_t)-1) {
+		if (!push_utf8_talloc(frame, &server, lp_cups_server(), &size)) {
 			return NULL;
 		}
 	} else {
@@ -73,10 +86,21 @@ static http_t *cups_connect(TALLOC_CTX *frame)
 	DEBUG(10, ("connecting to cups server %s:%d\n",
 		   server, port));
 
-	if ((http = httpConnect(server, port)) == NULL) {
+	gotalarm = 0;
+
+	if (timeout) {
+                CatchSignal(SIGALRM, SIGNAL_CAST gotalarm_sig);
+                alarm(timeout);
+        }
+
+	http = httpConnect(server, port);
+
+	CatchSignal(SIGALRM, SIGNAL_CAST SIG_IGN);
+        alarm(0);
+
+	if (http == NULL) {
 		DEBUG(0,("Unable to connect to CUPS server %s:%d - %s\n",
 			 server, port, strerror(errno)));
-		return NULL;
 	}
 
 	return http;
@@ -135,6 +159,7 @@ static bool cups_cache_reload_async(int fd)
 			  "printer-info"
 			};
 	bool ret = False;
+	size_t size;
 
 	DEBUG(5, ("reloading cups printcap cache\n"));
 
@@ -210,16 +235,22 @@ static bool cups_cache_reload_async(int fd)
 		while (attr != NULL && attr->group_tag == IPP_TAG_PRINTER) {
         		if (strcmp(attr->name, "printer-name") == 0 &&
 			    attr->value_tag == IPP_TAG_NAME) {
-				pull_utf8_talloc(frame,
+				if (!pull_utf8_talloc(frame,
 						&name,
-						attr->values[0].string.text);
+						attr->values[0].string.text,
+						&size)) {
+					goto out;
+				}
 			}
 
         		if (strcmp(attr->name, "printer-info") == 0 &&
 			    attr->value_tag == IPP_TAG_TEXT) {
-				pull_utf8_talloc(frame,
+				if (!pull_utf8_talloc(frame,
 						&info,
-						attr->values[0].string.text);
+						attr->values[0].string.text,
+						&size)) {
+					goto out;
+				}
 			}
 
         		attr = attr->next;
@@ -296,16 +327,22 @@ static bool cups_cache_reload_async(int fd)
 		while (attr != NULL && attr->group_tag == IPP_TAG_PRINTER) {
         		if (strcmp(attr->name, "printer-name") == 0 &&
 			    attr->value_tag == IPP_TAG_NAME) {
-				pull_utf8_talloc(frame,
+				if (!pull_utf8_talloc(frame,
 						&name,
-						attr->values[0].string.text);
+						attr->values[0].string.text,
+						&size)) {
+					goto out;
+				}
 			}
 
         		if (strcmp(attr->name, "printer-info") == 0 &&
 			    attr->value_tag == IPP_TAG_TEXT) {
-				pull_utf8_talloc(frame,
+				if (!pull_utf8_talloc(frame,
 						&info,
-						attr->values[0].string.text);
+						attr->values[0].string.text,
+						&size)) {
+					goto out;
+				}
 			}
 
         		attr = attr->next;
@@ -576,7 +613,7 @@ static int cups_job_delete(const char *sharename, const char *lprm_command, stru
 	cups_lang_t	*language = NULL;	/* Default language */
 	char *user = NULL;
 	char		uri[HTTP_MAX_URI]; /* printer-uri attribute */
-
+	size_t size;
 
 	DEBUG(5,("cups_job_delete(%s, %p (%d))\n", sharename, pjob, pjob->sysjob));
 
@@ -621,7 +658,7 @@ static int cups_job_delete(const char *sharename, const char *lprm_command, stru
 
 	ippAddString(request, IPP_TAG_OPERATION, IPP_TAG_URI, "job-uri", NULL, uri);
 
-	if (push_utf8_talloc(frame, &user, pjob->user) == (size_t)-1) {
+	if (!push_utf8_talloc(frame, &user, pjob->user, &size)) {
 		goto out;
 	}
 
@@ -673,7 +710,7 @@ static int cups_job_pause(int snum, struct printjob *pjob)
 	cups_lang_t	*language = NULL;	/* Default language */
 	char *user = NULL;
 	char		uri[HTTP_MAX_URI]; /* printer-uri attribute */
-
+	size_t size;
 
 	DEBUG(5,("cups_job_pause(%d, %p (%d))\n", snum, pjob, pjob->sysjob));
 
@@ -718,7 +755,7 @@ static int cups_job_pause(int snum, struct printjob *pjob)
 
 	ippAddString(request, IPP_TAG_OPERATION, IPP_TAG_URI, "job-uri", NULL, uri);
 
-	if (push_utf8_talloc(frame, &user, pjob->user) == (size_t)-1) {
+	if (!push_utf8_talloc(frame, &user, pjob->user, &size)) {
 		goto out;
 	}
 	ippAddString(request, IPP_TAG_OPERATION, IPP_TAG_NAME, "requesting-user-name",
@@ -769,7 +806,7 @@ static int cups_job_resume(int snum, struct printjob *pjob)
 	cups_lang_t	*language = NULL;	/* Default language */
 	char *user = NULL;
 	char		uri[HTTP_MAX_URI]; /* printer-uri attribute */
-
+	size_t size;
 
 	DEBUG(5,("cups_job_resume(%d, %p (%d))\n", snum, pjob, pjob->sysjob));
 
@@ -814,7 +851,7 @@ static int cups_job_resume(int snum, struct printjob *pjob)
 
 	ippAddString(request, IPP_TAG_OPERATION, IPP_TAG_URI, "job-uri", NULL, uri);
 
-	if (push_utf8_talloc(frame, &user, pjob->user) == (size_t)-1) {
+	if (!push_utf8_talloc(frame, &user, pjob->user, &size)) {
 		goto out;
 	}
 	ippAddString(request, IPP_TAG_OPERATION, IPP_TAG_NAME, "requesting-user-name",
@@ -873,6 +910,7 @@ static int cups_job_submit(int snum, struct printjob *pjob)
 	char *jobname = NULL;
 	char *cupsoptions = NULL;
 	char *filename = NULL;
+	size_t size;
 	char addr[INET6_ADDRSTRLEN];
 
 	DEBUG(5,("cups_job_submit(%d, %p (%d))\n", snum, pjob, pjob->sysjob));
@@ -915,7 +953,7 @@ static int cups_job_submit(int snum, struct printjob *pjob)
 	ippAddString(request, IPP_TAG_OPERATION, IPP_TAG_LANGUAGE,
         	     "attributes-natural-language", NULL, language->language);
 
-	if (push_utf8_talloc(frame, &printername, PRINTERNAME(snum)) == (size_t)-1) {
+	if (!push_utf8_talloc(frame, &printername, PRINTERNAME(snum), &size)) {
 		goto out;
 	}
 	slprintf(uri, sizeof(uri) - 1, "ipp://localhost/printers/%s",
@@ -924,7 +962,7 @@ static int cups_job_submit(int snum, struct printjob *pjob)
 	ippAddString(request, IPP_TAG_OPERATION, IPP_TAG_URI,
         	     "printer-uri", NULL, uri);
 
-	if (push_utf8_talloc(frame, &user, pjob->user) == (size_t)-1) {
+	if (!push_utf8_talloc(frame, &user, pjob->user, &size)) {
 		goto out;
 	}
 	ippAddString(request, IPP_TAG_OPERATION, IPP_TAG_NAME, "requesting-user-name",
@@ -939,7 +977,7 @@ static int cups_job_submit(int snum, struct printjob *pjob)
 	             "job-originating-host-name", NULL,
 		      clientname);
 
-	if (push_utf8_talloc(frame, &jobname, pjob->jobname) == (size_t)-1) {
+	if (!push_utf8_talloc(frame, &jobname, pjob->jobname, &size)) {
 		goto out;
 	}
 	new_jobname = talloc_asprintf(frame,
@@ -957,7 +995,7 @@ static int cups_job_submit(int snum, struct printjob *pjob)
 	 * add any options defined in smb.conf
 	 */
 
-	if (push_utf8_talloc(frame, &cupsoptions, lp_cups_options(snum)) == (size_t)-1) {
+	if (!push_utf8_talloc(frame, &cupsoptions, lp_cups_options(snum), &size)) {
 		goto out;
 	}
 	num_options = 0;
@@ -973,7 +1011,7 @@ static int cups_job_submit(int snum, struct printjob *pjob)
 
 	slprintf(uri, sizeof(uri) - 1, "/printers/%s", printername);
 
-	if (push_utf8_talloc(frame, &filename, pjob->filename) == (size_t)-1) {
+	if (!push_utf8_talloc(frame, &filename, pjob->filename, &size)) {
 		goto out;
 	}
 	if ((response = cupsDoFileRequest(http, request, uri, pjob->filename)) != NULL) {
@@ -1036,6 +1074,7 @@ static int cups_queue_get(const char *sharename,
 	time_t		job_time;	/* time-at-creation attribute */
 	ipp_jstate_t	job_status;	/* job-status attribute */
 	int		job_priority;	/* job-priority attribute */
+	size_t size;
 	static const char *jattrs[] =	/* Requested job attributes */
 			{
 			  "job-id",
@@ -1060,7 +1099,7 @@ static int cups_queue_get(const char *sharename,
 	   (which is basically what we do for non-cups printers ... using
 	   the lpq_command to get the queue listing). */
 
-	if (push_utf8_talloc(frame, &printername, lpq_command) == (size_t)-1) {
+	if (!push_utf8_talloc(frame, &printername, lpq_command, &size)) {
 		goto out;
 	}
 	DEBUG(5,("cups_queue_get(%s, %p, %p)\n", lpq_command, q, status));
@@ -1209,16 +1248,22 @@ static int cups_queue_get(const char *sharename,
 
         		if (strcmp(attr->name, "job-name") == 0 &&
 			    attr->value_tag == IPP_TAG_NAME) {
-				pull_utf8_talloc(frame,
+				if (!pull_utf8_talloc(frame,
 						&job_name,
-						attr->values[0].string.text);
+						attr->values[0].string.text,
+						&size)) {
+					goto out;
+				}
 			}
 
         		if (strcmp(attr->name, "job-originating-user-name") == 0 &&
 			    attr->value_tag == IPP_TAG_NAME) {
-				pull_utf8_talloc(frame,
+				if (!pull_utf8_talloc(frame,
 						&user_name,
-						attr->values[0].string.text);
+						attr->values[0].string.text,
+						&size)) {
+					goto out;
+				}
 			}
 
         		attr = attr->next;
@@ -1316,7 +1361,11 @@ static int cups_queue_get(const char *sharename,
         if ((attr = ippFindAttribute(response, "printer-state-message",
 	                             IPP_TAG_TEXT)) != NULL) {
 		char *msg = NULL;
-		pull_utf8_talloc(frame, &msg, attr->values[0].string.text);
+		if (!pull_utf8_talloc(frame, &msg,
+				attr->values[0].string.text,
+				&size)) {
+			goto out;
+		}
 	        fstrcpy(status->message, msg);
 	}
 
@@ -1356,7 +1405,7 @@ static int cups_queue_pause(int snum)
 	char *printername = NULL;
 	char *username = NULL;
 	char		uri[HTTP_MAX_URI]; /* printer-uri attribute */
-
+	size_t size;
 
 	DEBUG(5,("cups_queue_pause(%d)\n", snum));
 
@@ -1397,7 +1446,7 @@ static int cups_queue_pause(int snum)
 	ippAddString(request, IPP_TAG_OPERATION, IPP_TAG_LANGUAGE,
         	     "attributes-natural-language", NULL, language->language);
 
-	if (push_utf8_talloc(frame, &printername, PRINTERNAME(snum)) == (size_t)-1) {
+	if (!push_utf8_talloc(frame, &printername, PRINTERNAME(snum), &size)) {
 		goto out;
 	}
 	slprintf(uri, sizeof(uri) - 1, "ipp://localhost/printers/%s",
@@ -1405,7 +1454,7 @@ static int cups_queue_pause(int snum)
 
 	ippAddString(request, IPP_TAG_OPERATION, IPP_TAG_URI, "printer-uri", NULL, uri);
 
-	if (push_utf8_talloc(frame, &username, current_user_info.unix_name) == (size_t)-1) {
+	if (!push_utf8_talloc(frame, &username, current_user_info.unix_name, &size)) {
 		goto out;
 	}
 	ippAddString(request, IPP_TAG_OPERATION, IPP_TAG_NAME, "requesting-user-name",
@@ -1457,7 +1506,7 @@ static int cups_queue_resume(int snum)
 	char *printername = NULL;
 	char *username = NULL;
 	char		uri[HTTP_MAX_URI]; /* printer-uri attribute */
-
+	size_t size;
 
 	DEBUG(5,("cups_queue_resume(%d)\n", snum));
 
@@ -1498,7 +1547,7 @@ static int cups_queue_resume(int snum)
 	ippAddString(request, IPP_TAG_OPERATION, IPP_TAG_LANGUAGE,
         	     "attributes-natural-language", NULL, language->language);
 
-	if (push_utf8_talloc(frame, &printername, PRINTERNAME(snum)) == (size_t)-1) {
+	if (!push_utf8_talloc(frame, &printername, PRINTERNAME(snum), &size)) {
 		goto out;
 	}
 	slprintf(uri, sizeof(uri) - 1, "ipp://localhost/printers/%s",
@@ -1506,7 +1555,7 @@ static int cups_queue_resume(int snum)
 
 	ippAddString(request, IPP_TAG_OPERATION, IPP_TAG_URI, "printer-uri", NULL, uri);
 
-	if (push_utf8_talloc(frame, &username, current_user_info.unix_name) == (size_t)-1) {
+	if (!push_utf8_talloc(frame, &username, current_user_info.unix_name, &size)) {
 		goto out;
 	}
 	ippAddString(request, IPP_TAG_OPERATION, IPP_TAG_NAME, "requesting-user-name",
@@ -1577,6 +1626,7 @@ bool cups_pull_comment_location(NT_PRINTER_INFO_LEVEL_2 *printer)
 			  "printer-location"
 			};
 	bool ret = False;
+	size_t size;
 
 	DEBUG(5, ("pulling %s location\n", printer->sharename));
 
@@ -1608,7 +1658,7 @@ bool cups_pull_comment_location(NT_PRINTER_INFO_LEVEL_2 *printer)
                      "attributes-natural-language", NULL, language->language);
 
 	if (lp_cups_server() != NULL && strlen(lp_cups_server()) > 0) {
-		if (push_utf8_talloc(frame, &server, lp_cups_server()) == (size_t)-1) {
+		if (!push_utf8_talloc(frame, &server, lp_cups_server(), &size)) {
 			goto out;
 		}
 	} else {
@@ -1617,7 +1667,7 @@ bool cups_pull_comment_location(NT_PRINTER_INFO_LEVEL_2 *printer)
 	if (server) {
 		goto out;
 	}
-	if (push_utf8_talloc(frame, &sharename, printer->sharename) == (size_t)-1) {
+	if (!push_utf8_talloc(frame, &sharename, printer->sharename, &size)) {
 		goto out;
 	}
 	slprintf(uri, sizeof(uri) - 1, "ipp://%s/printers/%s",
@@ -1659,9 +1709,12 @@ bool cups_pull_comment_location(NT_PRINTER_INFO_LEVEL_2 *printer)
 		while ( attr && (attr->group_tag == IPP_TAG_PRINTER) ) {
         		if (strcmp(attr->name, "printer-name") == 0 &&
 			    attr->value_tag == IPP_TAG_NAME) {
-				pull_utf8_talloc(frame,
+				if (!pull_utf8_talloc(frame,
 						&name,
-						attr->values[0].string.text);
+						attr->values[0].string.text,
+						&size)) {
+					goto out;
+				}
 			}
 
 			/* Grab the comment if we don't have one */
@@ -1670,9 +1723,12 @@ bool cups_pull_comment_location(NT_PRINTER_INFO_LEVEL_2 *printer)
 			     && !strlen(printer->comment) )
 			{
 				char *comment = NULL;
-				pull_utf8_talloc(frame,
-					&comment,
-					attr->values[0].string.text);
+				if (!pull_utf8_talloc(frame,
+						&comment,
+						attr->values[0].string.text,
+						&size)) {
+					goto out;
+				}
 				DEBUG(5,("cups_pull_comment_location: Using cups comment: %s\n",
 					 comment));
 			    	strlcpy(printer->comment,
@@ -1686,9 +1742,12 @@ bool cups_pull_comment_location(NT_PRINTER_INFO_LEVEL_2 *printer)
 			     && !strlen(printer->location) )
 			{
 				char *location = NULL;
-				pull_utf8_talloc(frame,
-					&location,
-					attr->values[0].string.text);
+				if (!pull_utf8_talloc(frame,
+						&location,
+						attr->values[0].string.text,
+						&size)) {
+					goto out;
+				}
 				DEBUG(5,("cups_pull_comment_location: Using cups location: %s\n",
 					 location));
 			    	strlcpy(printer->location,
