@@ -377,7 +377,7 @@ NTSTATUS _netr_ServerReqChallenge(pipes_struct *p,
 				  struct netr_ServerReqChallenge *r)
 {
 	if (!p->dc) {
-		p->dc = TALLOC_ZERO_P(p->pipe_state_mem_ctx, struct dcinfo);
+		p->dc = TALLOC_ZERO_P(p, struct dcinfo);
 		if (!p->dc) {
 			return NT_STATUS_NO_MEMORY;
 		}
@@ -472,59 +472,25 @@ NTSTATUS _netr_ServerAuthenticate2(pipes_struct *p,
 {
 	NTSTATUS status;
 	uint32_t srv_flgs;
-	/* r->in.negotiate_flags is an aliased pointer to r->out.negotiate_flags,
-	 * so use a copy to avoid destroying the client values. */
-	uint32_t in_neg_flags = *r->in.negotiate_flags;
 	struct netr_Credential srv_chal_out;
 
-	/* According to Microsoft (see bugid #6099)
-	 * Windows 7 looks at the negotiate_flags
-	 * returned in this structure *even if the
-	 * call fails with access denied* ! So in order
-	 * to allow Win7 to connect to a Samba NT style
-	 * PDC we set the flags before we know if it's
-	 * an error or not.
-	 */
-
-	/* 0x000001ff */
-	srv_flgs = NETLOGON_NEG_ACCOUNT_LOCKOUT |
-		   NETLOGON_NEG_PERSISTENT_SAMREPL |
-		   NETLOGON_NEG_ARCFOUR |
-		   NETLOGON_NEG_PROMOTION_COUNT |
-		   NETLOGON_NEG_CHANGELOG_BDC |
-		   NETLOGON_NEG_FULL_SYNC_REPL |
-		   NETLOGON_NEG_MULTIPLE_SIDS |
-		   NETLOGON_NEG_REDO |
-		   NETLOGON_NEG_PASSWORD_CHANGE_REFUSAL;
-
-	/* Ensure we support strong (128-bit) keys. */
-	if (in_neg_flags & NETLOGON_NEG_128BIT) {
-		srv_flgs |= NETLOGON_NEG_128BIT;
-	}
-
-	if (lp_server_schannel() != false) {
-		srv_flgs |= NETLOGON_NEG_SCHANNEL;
-	}
-
 	/* We use this as the key to store the creds: */
 	/* r->in.computer_name */
 
 	if (!p->dc || !p->dc->challenge_sent) {
 		DEBUG(0,("_netr_ServerAuthenticate2: no challenge sent to client %s\n",
 			r->in.computer_name));
-		status = NT_STATUS_ACCESS_DENIED;
-		goto out;
+		return NT_STATUS_ACCESS_DENIED;
 	}
 
 	if ( (lp_server_schannel() == true) &&
-	     ((in_neg_flags & NETLOGON_NEG_SCHANNEL) == 0) ) {
+	     ((*r->in.negotiate_flags & NETLOGON_NEG_SCHANNEL) == 0) ) {
 
 		/* schannel must be used, but client did not offer it. */
 		DEBUG(0,("_netr_ServerAuthenticate2: schannel required but client failed "
 			"to offer it. Client was %s\n",
 			r->in.account_name));
-		status = NT_STATUS_ACCESS_DENIED;
-		goto out;
+		return NT_STATUS_ACCESS_DENIED;
 	}
 
 	status = get_md4pw((char *)p->dc->mach_pw,
@@ -535,12 +501,11 @@ NTSTATUS _netr_ServerAuthenticate2(pipes_struct *p,
 			"account %s: %s\n",
 			r->in.account_name, nt_errstr(status) ));
 		/* always return NT_STATUS_ACCESS_DENIED */
-		status = NT_STATUS_ACCESS_DENIED;
-		goto out;
+		return NT_STATUS_ACCESS_DENIED;
 	}
 
 	/* From the client / server challenges and md4 password, generate sess key */
-	creds_server_init(in_neg_flags,
+	creds_server_init(*r->in.negotiate_flags,
 			p->dc,
 			&p->dc->clnt_chal,	/* Stored client chal. */
 			&p->dc->srv_chal,	/* Stored server chal. */
@@ -553,13 +518,28 @@ NTSTATUS _netr_ServerAuthenticate2(pipes_struct *p,
 			"request from client %s machine account %s\n",
 			r->in.computer_name,
 			r->in.account_name));
-		status = NT_STATUS_ACCESS_DENIED;
-		goto out;
+		return NT_STATUS_ACCESS_DENIED;
+	}
+
+	/* 0x000001ff */
+	srv_flgs = NETLOGON_NEG_ACCOUNT_LOCKOUT |
+		   NETLOGON_NEG_PERSISTENT_SAMREPL |
+		   NETLOGON_NEG_ARCFOUR |
+		   NETLOGON_NEG_PROMOTION_COUNT |
+		   NETLOGON_NEG_CHANGELOG_BDC |
+		   NETLOGON_NEG_FULL_SYNC_REPL |
+		   NETLOGON_NEG_MULTIPLE_SIDS |
+		   NETLOGON_NEG_REDO |
+		   NETLOGON_NEG_PASSWORD_CHANGE_REFUSAL;
+
+	if (lp_server_schannel() != false) {
+		srv_flgs |= NETLOGON_NEG_SCHANNEL;
 	}
 
 	/* set up the LSA AUTH 2 response */
 	memcpy(r->out.return_credentials->data, &srv_chal_out.data,
 	       sizeof(r->out.return_credentials->data));
+	*r->out.negotiate_flags = srv_flgs;
 
 	fstrcpy(p->dc->mach_acct, r->in.account_name);
 	fstrcpy(p->dc->remote_machine, r->in.computer_name);
@@ -573,12 +553,8 @@ NTSTATUS _netr_ServerAuthenticate2(pipes_struct *p,
 					    r->in.computer_name,
 					    p->dc);
 	unbecome_root();
-	status = NT_STATUS_OK;
 
-  out:
-
-	*r->out.negotiate_flags = srv_flgs;
-	return status;
+	return NT_STATUS_OK;
 }
 
 /*************************************************************************
@@ -615,9 +591,8 @@ NTSTATUS _netr_ServerPasswordSet(pipes_struct *p,
 	if (!p->dc) {
 		/* Restore the saved state of the netlogon creds. */
 		become_root();
-		ret = secrets_restore_schannel_session_info(p->pipe_state_mem_ctx,
-							remote_machine,
-							&p->dc);
+		ret = secrets_restore_schannel_session_info(p, remote_machine,
+							    &p->dc);
 		unbecome_root();
 		if (!ret) {
 			return NT_STATUS_INVALID_HANDLE;
@@ -646,9 +621,7 @@ NTSTATUS _netr_ServerPasswordSet(pipes_struct *p,
 	}
 
 	become_root();
-	secrets_store_schannel_session_info(p->pipe_state_mem_ctx,
-						remote_machine,
-						p->dc);
+	secrets_store_schannel_session_info(p, remote_machine, p->dc);
 	ret = pdb_getsampwnam(sampass, p->dc->mach_acct);
 	unbecome_root();
 
@@ -715,7 +688,7 @@ NTSTATUS _netr_ServerPasswordSet(pipes_struct *p,
 	/* set up the LSA Server Password Set response */
 
 	memcpy(r->out.return_authenticator, &cred_out,
-	       sizeof(*(r->out.return_authenticator)));
+	       sizeof(r->out.return_authenticator));
 
 	TALLOC_FREE(sampass);
 	return status;
@@ -738,9 +711,6 @@ NTSTATUS _netr_LogonSamLogoff(pipes_struct *p,
 	}
 
 
-	if (!get_valid_user_struct(p->vuid))
-		return NT_STATUS_NO_SUCH_USER;
-
 	/* Using the remote machine name for the creds store: */
 	/* r->in.computer_name */
 
@@ -749,9 +719,8 @@ NTSTATUS _netr_LogonSamLogoff(pipes_struct *p,
 		bool ret;
 
 		become_root();
-		ret = secrets_restore_schannel_session_info(p->pipe_state_mem_ctx,
-							    r->in.computer_name,
-							    &p->dc);
+		ret = secrets_restore_schannel_session_info(
+			p, r->in.computer_name, &p->dc);
 		unbecome_root();
 		if (!ret) {
 			return NT_STATUS_INVALID_HANDLE;
@@ -772,44 +741,12 @@ NTSTATUS _netr_LogonSamLogoff(pipes_struct *p,
 
 	/* We must store the creds state after an update. */
 	become_root();
-	secrets_store_schannel_session_info(p->pipe_state_mem_ctx,
-					    r->in.computer_name,
-					    p->dc);
+	secrets_store_schannel_session_info(p, r->in.computer_name, p->dc);
 	unbecome_root();
 
 	return NT_STATUS_OK;
 }
 
-/*******************************************************************
- gets a domain user's groups from their already-calculated NT_USER_TOKEN
- ********************************************************************/
-
-static NTSTATUS nt_token_to_group_list(TALLOC_CTX *mem_ctx,
-				       const DOM_SID *domain_sid,
-				       size_t num_sids,
-				       const DOM_SID *sids,
-				       int *numgroups, DOM_GID **pgids)
-{
-	int i;
-
-	*numgroups=0;
-	*pgids = NULL;
-
-	for (i=0; i<num_sids; i++) {
-		DOM_GID gid;
-		if (!sid_peek_check_rid(domain_sid, &sids[i], &gid.g_rid)) {
-			continue;
-		}
-		gid.attr = (SE_GROUP_MANDATORY|SE_GROUP_ENABLED_BY_DEFAULT|
-			    SE_GROUP_ENABLED);
-		ADD_TO_ARRAY(mem_ctx, DOM_GID, gid, pgids, numgroups);
-		if (*pgids == NULL) {
-			return NT_STATUS_NO_MEMORY;
-		}
-	}
-	return NT_STATUS_OK;
-}
-
 /*************************************************************************
  _netr_LogonSamLogon
  *************************************************************************/
@@ -823,8 +760,8 @@ NTSTATUS _netr_LogonSamLogon(pipes_struct *p,
 	fstring nt_username, nt_domain, nt_workstation;
 	auth_usersupplied_info *user_info = NULL;
 	auth_serversupplied_info *server_info = NULL;
-	struct samu *sampw;
 	struct auth_context *auth_context = NULL;
+	uint8_t pipe_session_key[16];
 	bool process_creds = true;
 
 	switch (p->hdr_req.opnum) {
@@ -859,27 +796,20 @@ NTSTATUS _netr_LogonSamLogon(pipes_struct *p,
 		return NT_STATUS_ACCESS_DENIED;
 	}
 
-	if (!get_valid_user_struct(p->vuid))
-		return NT_STATUS_NO_SUCH_USER;
-
 	if (process_creds) {
-		fstring remote_machine;
 
 		/* Get the remote machine name for the creds store. */
 		/* Note this is the remote machine this request is coming from (member server),
 		   not neccessarily the workstation name the user is logging onto.
 		*/
 
-		fstrcpy(remote_machine, r->in.computer_name);
-
 		if (!p->dc) {
 			/* Restore the saved state of the netlogon creds. */
 			bool ret;
 
 			become_root();
-			ret = secrets_restore_schannel_session_info(p->pipe_state_mem_ctx,
-					remote_machine,
-					&p->dc);
+			ret = secrets_restore_schannel_session_info(
+				p, r->in.computer_name, &p->dc);
 			unbecome_root();
 			if (!ret) {
 				return NT_STATUS_INVALID_HANDLE;
@@ -894,15 +824,13 @@ NTSTATUS _netr_LogonSamLogon(pipes_struct *p,
 		if (!netlogon_creds_server_step(p->dc, r->in.credential,  r->out.return_authenticator)) {
 			DEBUG(2,("_netr_LogonSamLogon: creds_server_step failed. Rejecting auth "
 				"request from client %s machine account %s\n",
-				remote_machine, p->dc->mach_acct ));
+				r->in.computer_name, p->dc->mach_acct ));
 			return NT_STATUS_INVALID_PARAMETER;
 		}
 
 		/* We must store the creds state after an update. */
 		become_root();
-		secrets_store_schannel_session_info(p->pipe_state_mem_ctx,
-					remote_machine,
-					p->dc);
+		secrets_store_schannel_session_info(p, r->in.computer_name, p->dc);
 		unbecome_root();
 	}
 
@@ -1041,160 +969,19 @@ NTSTATUS _netr_LogonSamLogon(pipes_struct *p,
            the SAM Local Security Authority should record that the user is
            logged in to the domain.  */
 
-	{
-		DOM_GID *gids = NULL;
-		const DOM_SID *user_sid = NULL;
-		const DOM_SID *group_sid = NULL;
-		DOM_SID domain_sid;
-		uint32 user_rid, group_rid;
-
-		int num_gids = 0;
-		const char *my_name;
-
-		struct netr_UserSessionKey user_session_key;
-		struct netr_LMSessionKey lm_session_key;
-		unsigned char pipe_session_key[16];
-
-		NTTIME last_logon, last_logoff, acct_expiry, last_password_change;
-		NTTIME allow_password_change, force_password_change;
-		struct samr_RidWithAttributeArray groups;
-		int i;
-		struct dom_sid2 *sid = NULL;
-
-		ZERO_STRUCT(user_session_key);
-		ZERO_STRUCT(lm_session_key);
-
-		sampw = server_info->sam_account;
-
-		user_sid = pdb_get_user_sid(sampw);
-		group_sid = pdb_get_group_sid(sampw);
-
-		if ((user_sid == NULL) || (group_sid == NULL)) {
-			DEBUG(1, ("_netr_LogonSamLogon: User without group or user SID\n"));
-			return NT_STATUS_UNSUCCESSFUL;
+	if (process_creds) {
+		/* Get the pipe session key from the creds. */
+		memcpy(pipe_session_key, p->dc->sess_key, 16);
+	} else {
+		/* Get the pipe session key from the schannel. */
+		if ((p->auth.auth_type != PIPE_AUTH_TYPE_SCHANNEL)
+		    || (p->auth.a_u.schannel_auth == NULL)) {
+			return NT_STATUS_INVALID_HANDLE;
 		}
-
-		sid_copy(&domain_sid, user_sid);
-		sid_split_rid(&domain_sid, &user_rid);
-
-		sid = sid_dup_talloc(p->mem_ctx, &domain_sid);
-		if (!sid) {
-			return NT_STATUS_NO_MEMORY;
-		}
-
-		if (!sid_peek_check_rid(&domain_sid, group_sid, &group_rid)) {
-			DEBUG(1, ("_netr_LogonSamLogon: user %s\\%s has user sid "
-				  "%s\n but group sid %s.\n"
-				  "The conflicting domain portions are not "
-				  "supported for NETLOGON calls\n",
-				  pdb_get_domain(sampw),
-				  pdb_get_username(sampw),
-				  sid_string_dbg(user_sid),
-				  sid_string_dbg(group_sid)));
-			return NT_STATUS_UNSUCCESSFUL;
-		}
-
-		if(server_info->login_server) {
-		        my_name = server_info->login_server;
-		} else {
-		        my_name = global_myname();
-		}
-
-		status = nt_token_to_group_list(p->mem_ctx, &domain_sid,
-						server_info->num_sids,
-						server_info->sids,
-						&num_gids, &gids);
-
-		if (!NT_STATUS_IS_OK(status)) {
-			return status;
-		}
-
-		if (server_info->user_session_key.length) {
-			memcpy(user_session_key.key,
-			       server_info->user_session_key.data,
-			       MIN(sizeof(user_session_key.key),
-				   server_info->user_session_key.length));
-			if (process_creds) {
-				/* Get the pipe session key from the creds. */
-				memcpy(pipe_session_key, p->dc->sess_key, 16);
-			} else {
-				/* Get the pipe session key from the schannel. */
-				if (p->auth.auth_type != PIPE_AUTH_TYPE_SCHANNEL || p->auth.a_u.schannel_auth == NULL) {
-					return NT_STATUS_INVALID_HANDLE;
-				}
-				memcpy(pipe_session_key, p->auth.a_u.schannel_auth->sess_key, 16);
-			}
-			SamOEMhash(user_session_key.key, pipe_session_key, 16);
-			memset(pipe_session_key, '\0', 16);
-		}
-		if (server_info->lm_session_key.length) {
-			memcpy(lm_session_key.key,
-			       server_info->lm_session_key.data,
-			       MIN(sizeof(lm_session_key.key),
-				   server_info->lm_session_key.length));
-			if (process_creds) {
-				/* Get the pipe session key from the creds. */
-				memcpy(pipe_session_key, p->dc->sess_key, 16);
-			} else {
-				/* Get the pipe session key from the schannel. */
-				if (p->auth.auth_type != PIPE_AUTH_TYPE_SCHANNEL || p->auth.a_u.schannel_auth == NULL) {
-					return NT_STATUS_INVALID_HANDLE;
-				}
-				memcpy(pipe_session_key, p->auth.a_u.schannel_auth->sess_key, 16);
-			}
-			SamOEMhash(lm_session_key.key, pipe_session_key, 8);
-			memset(pipe_session_key, '\0', 16);
-		}
-
-		groups.count = num_gids;
-		groups.rids = TALLOC_ARRAY(p->mem_ctx, struct samr_RidWithAttribute,
-					   groups.count);
-		if (!groups.rids) {
-			return NT_STATUS_NO_MEMORY;
-		}
-
-		for (i=0; i < groups.count; i++) {
-			groups.rids[i].rid = gids[i].g_rid;
-			groups.rids[i].attributes = gids[i].attr;
-		}
-
-		unix_to_nt_time(&last_logon, pdb_get_logon_time(sampw));
-		unix_to_nt_time(&last_logoff, get_time_t_max());
-		unix_to_nt_time(&acct_expiry, get_time_t_max());
-		unix_to_nt_time(&last_password_change, pdb_get_pass_last_set_time(sampw));
-		unix_to_nt_time(&allow_password_change, pdb_get_pass_can_change_time(sampw));
-		unix_to_nt_time(&force_password_change, pdb_get_pass_must_change_time(sampw));
-
-		init_netr_SamInfo3(sam3,
-				   last_logon,
-				   last_logoff,
-				   acct_expiry,
-				   last_password_change,
-				   allow_password_change,
-				   force_password_change,
-				   talloc_strdup(p->mem_ctx, pdb_get_username(sampw)),
-				   talloc_strdup(p->mem_ctx, pdb_get_fullname(sampw)),
-				   talloc_strdup(p->mem_ctx, pdb_get_logon_script(sampw)),
-				   talloc_strdup(p->mem_ctx, pdb_get_profile_path(sampw)),
-				   talloc_strdup(p->mem_ctx, pdb_get_homedir(sampw)),
-				   talloc_strdup(p->mem_ctx, pdb_get_dir_drive(sampw)),
-				   0, /* logon_count */
-				   0, /* bad_password_count */
-				   user_rid,
-				   group_rid,
-				   groups,
-				   NETLOGON_EXTRA_SIDS,
-				   user_session_key,
-				   my_name,
-				   talloc_strdup(p->mem_ctx, pdb_get_domain(sampw)),
-				   sid,
-				   lm_session_key,
-				   pdb_get_acct_ctrl(sampw),
-				   0, /* sidcount */
-				   NULL); /* struct netr_SidAttr *sids */
-		ZERO_STRUCT(user_session_key);
-		ZERO_STRUCT(lm_session_key);
+		memcpy(pipe_session_key, p->auth.a_u.schannel_auth->sess_key, 16);
 	}
+
+	status = serverinfo_to_SamInfo3(server_info, pipe_session_key, sam3);
 	TALLOC_FREE(server_info);
 	return status;
 }
@@ -1375,10 +1162,11 @@ WERROR _netr_DsRGetDCName(pipes_struct *p,
 /****************************************************************
 ****************************************************************/
 
-NTSTATUS _netr_LogonGetCapabilities(pipes_struct *p,
-				    struct netr_LogonGetCapabilities *r)
+WERROR _netr_NETRLOGONDUMMYROUTINE1(pipes_struct *p,
+				    struct netr_NETRLOGONDUMMYROUTINE1 *r)
 {
-	return NT_STATUS_NOT_IMPLEMENTED;
+	p->rng_fault_state = true;
+	return WERR_NOT_SUPPORTED;
 }
 
 /****************************************************************
