@@ -589,7 +589,7 @@ static NTSTATUS close_normal_file(files_struct *fsp, enum file_close_type close_
 	}
 
 	DEBUG(2,("%s closed file %s (numopen=%d) %s\n",
-		conn->user,fsp->fsp_name,
+		conn->server_info->unix_name,fsp->fsp_name,
 		conn->num_files_open - 1,
 		nt_errstr(status) ));
 
@@ -710,20 +710,6 @@ static NTSTATUS close_directory(files_struct *fsp, enum file_close_type close_ty
 	return status;
 }
 
-/****************************************************************************
- Close a 'stat file' opened internally.
-****************************************************************************/
-  
-static NTSTATUS close_stat(files_struct *fsp)
-{
-	/*
-	 * Do the code common to files and directories.
-	 */
-	close_filestruct(fsp);
-	file_free(fsp);
-	return NT_STATUS_OK;
-}
-
 /****************************************************************************
  Close a files_struct.
 ****************************************************************************/
@@ -735,8 +721,6 @@ NTSTATUS close_file(files_struct *fsp, enum file_close_type close_type)
 
 	if(fsp->is_directory) {
 		status = close_directory(fsp, close_type);
-	} else if (fsp->is_stat) {
-		status = close_stat(fsp);
 	} else if (fsp->fake_file_handle != NULL) {
 		status = close_fake_file(fsp);
 	} else {
