@@ -26,8 +26,6 @@
 
 #include "includes.h"
 
-static bool scan_directory(connection_struct *conn, const char *path,
-			   char *name, char **found_name);
 static NTSTATUS build_stream_path(TALLOC_CTX *mem_ctx,
 				  connection_struct *conn,
 				  const char *orig_path,
@@ -101,8 +99,7 @@ get any fatal errors that should immediately terminate the calling
 SMB processing whilst resolving.
 
 If the saved_last_component != 0, then the unmodified last component
-of the pathname is returned there. This is used in an exceptional
-case in reply_mv (so far). If saved_last_component == 0 then nothing
+of the pathname is returned there. If saved_last_component == 0 then nothing
 is returned there.
 
 If last_component_wcard is true then a MS wildcard was detected and
@@ -195,39 +192,11 @@ NTSTATUS unix_convert(TALLOC_CTX *ctx,
 		return result;
 	}
 
-	/*
-	 * Ensure saved_last_component is valid even if file exists.
-	 */
-
-	if(pp_saved_last_component) {
-		end = strrchr_m(orig_path, '/');
-		if (end) {
-			*pp_saved_last_component = talloc_strdup(ctx, end + 1);
-		} else {
-			*pp_saved_last_component = talloc_strdup(ctx,
-							orig_path);
-		}
-	}
-
 	if (!(name = talloc_strdup(ctx, orig_path))) {
 		DEBUG(0, ("talloc_strdup failed\n"));
 		return NT_STATUS_NO_MEMORY;
 	}
 
-	if (!lp_posix_pathnames()) {
-		stream = strchr_m(name, ':');
-
-		if (stream != NULL) {
-			char *tmp = talloc_strdup(ctx, stream);
-			if (tmp == NULL) {
-				TALLOC_FREE(name);
-				return NT_STATUS_NO_MEMORY;
-			}
-			*stream = '\0';
-			stream = tmp;
-		}
-	}
-
 	/*
 	 * Large directory fix normalization. If we're case sensitive, and
 	 * the case preserving parameters are set to "no", normalize the case of
@@ -242,6 +211,34 @@ NTSTATUS unix_convert(TALLOC_CTX *ctx,
 		strnorm(name, lp_defaultcase(SNUM(conn)));
 	}
 
+	/*
+	 * Ensure saved_last_component is valid even if file exists.
+	 */
+
+	if(pp_saved_last_component) {
+		end = strrchr_m(name, '/');
+		if (end) {
+			*pp_saved_last_component = talloc_strdup(ctx, end + 1);
+		} else {
+			*pp_saved_last_component = talloc_strdup(ctx,
+							name);
+		}
+	}
+
+	if (!lp_posix_pathnames()) {
+		stream = strchr_m(name, ':');
+
+		if (stream != NULL) {
+			char *tmp = talloc_strdup(ctx, stream);
+			if (tmp == NULL) {
+				TALLOC_FREE(name);
+				return NT_STATUS_NO_MEMORY;
+			}
+			*stream = '\0';
+			stream = tmp;
+		}
+	}
+
 	start = name;
 
 	/* If we're providing case insentive semantics or
@@ -434,8 +431,9 @@ NTSTATUS unix_convert(TALLOC_CTX *ctx,
 			 */
 
 			if (name_has_wildcard ||
-			    !scan_directory(conn, dirpath,
-				    start, &found_name)) {
+			    (SMB_VFS_GET_REAL_FILENAME(
+				     conn, dirpath, start,
+				     talloc_tos(), &found_name) == -1)) {
 				char *unmangled;
 
 				if (end) {
@@ -769,15 +767,15 @@ static bool fname_equal(const char *name1, const char *name2,
  If the name looks like a mangled name then try via the mangling functions
 ****************************************************************************/
 
-static bool scan_directory(connection_struct *conn, const char *path,
-			   char *name, char **found_name)
+int get_real_filename(connection_struct *conn, const char *path,
+		      const char *name, TALLOC_CTX *mem_ctx,
+		      char **found_name)
 {
 	struct smb_Dir *cur_dir;
 	const char *dname;
 	bool mangled;
 	char *unmangled_name = NULL;
 	long curpos;
-	TALLOC_CTX *ctx = talloc_tos();
 
 	mangled = mangle_is_mangled(name, conn->params);
 
@@ -792,7 +790,7 @@ static bool scan_directory(connection_struct *conn, const char *path,
 	 */
 	if (!mangled && !(conn->fs_capabilities & FILE_CASE_SENSITIVE_SEARCH)) {
 		errno = ENOENT;
-		return False;
+		return -1;
 	}
 
 	/*
@@ -811,10 +809,9 @@ static bool scan_directory(connection_struct *conn, const char *path,
 	 */
 
 	if (mangled && !conn->case_sensitive) {
-		mangled = !mangle_lookup_name_from_8_3(ctx,
-						name,
-						&unmangled_name,
-						conn->params);
+		mangled = !mangle_lookup_name_from_8_3(talloc_tos(), name,
+						       &unmangled_name,
+						       conn->params);
 		if (!mangled) {
 			/* Name is now unmangled. */
 			name = unmangled_name;
@@ -825,7 +822,7 @@ static bool scan_directory(connection_struct *conn, const char *path,
 	if (!(cur_dir = OpenDir(talloc_tos(), conn, path, NULL, 0))) {
 		DEBUG(3,("scan dir didn't open dir [%s]\n",path));
 		TALLOC_FREE(unmangled_name);
-		return(False);
+		return -1;
 	}
 
 	/* now scan for matching names */
@@ -851,21 +848,21 @@ static bool scan_directory(connection_struct *conn, const char *path,
 		if ((mangled && mangled_equal(name,dname,conn->params)) ||
 			fname_equal(name, dname, conn->case_sensitive)) {
 			/* we've found the file, change it's name and return */
-			*found_name = talloc_strdup(ctx,dname);
+			*found_name = talloc_strdup(mem_ctx, dname);
 			TALLOC_FREE(unmangled_name);
 			TALLOC_FREE(cur_dir);
 			if (!*found_name) {
 				errno = ENOMEM;
-				return False;
+				return -1;
 			}
-			return(True);
+			return 0;
 		}
 	}
 
 	TALLOC_FREE(unmangled_name);
 	TALLOC_FREE(cur_dir);
 	errno = ENOENT;
-	return False;
+	return -1;
 }
 
 static NTSTATUS build_stream_path(TALLOC_CTX *mem_ctx,
