@@ -402,9 +402,7 @@ void file_free(files_struct *fsp)
 
 	string_free(&fsp->fsp_name);
 
-	if (fsp->fake_file_handle) {
-		destroy_fake_file_handle(&fsp->fake_file_handle);
-	}
+	TALLOC_FREE(fsp->fake_file_handle);
 
 	if (fsp->fh->ref_count == 1) {
 		SAFE_FREE(fsp->fh);
@@ -505,10 +503,10 @@ void file_chain_reset(void)
 ****************************************************************************/
 
 void dup_file_fsp(files_struct *from,
-			uint32 access_mask,
-			uint32 share_access,
-			uint32 create_options,
-	      		files_struct *to)
+				uint32 access_mask,
+				uint32 share_access,
+				uint32 create_options,
+		      		files_struct *to)
 {
 	SAFE_FREE(to->fh);
 
@@ -534,7 +532,6 @@ void dup_file_fsp(files_struct *from,
 	to->print_file = from->print_file;
 	to->modified = from->modified;
 	to->is_directory = from->is_directory;
-	to->is_stat = from->is_stat;
 	to->aio_write_behind = from->aio_write_behind;
         string_set(&to->fsp_name,from->fsp_name);
 }
