@@ -313,11 +313,11 @@ static NTSTATUS idmap_tdb_alloc_init( const char *params )
 {
 	NTSTATUS ret;
 	TALLOC_CTX *ctx;
-	const char *range;
 	uid_t low_uid = 0;
 	uid_t high_uid = 0;
 	gid_t low_gid = 0;
 	gid_t high_gid = 0;
+	uint32_t low_id;
 
 	/* use our own context here */
 	ctx = talloc_new(NULL);
@@ -335,67 +335,47 @@ static NTSTATUS idmap_tdb_alloc_init( const char *params )
 	talloc_free(ctx);
 
 	/* load ranges */
-	idmap_tdb_state.low_uid = 0;
-	idmap_tdb_state.high_uid = 0;
-	idmap_tdb_state.low_gid = 0;
-	idmap_tdb_state.high_gid = 0;
 
-	range = lp_parm_const_string(-1, "idmap alloc config", "range", NULL);
-	if (range && range[0]) {
-		unsigned low_id, high_id;
-
-		if (sscanf(range, "%u - %u", &low_id, &high_id) == 2) {
-			if (low_id < high_id) {
-				idmap_tdb_state.low_gid = idmap_tdb_state.low_uid = low_id;
-				idmap_tdb_state.high_gid = idmap_tdb_state.high_uid = high_id;
-			} else {
-				DEBUG(1, ("ERROR: invalid idmap alloc range [%s]", range));
-			}
-		} else {
-			DEBUG(1, ("ERROR: invalid syntax for idmap alloc config:range [%s]", range));
-		}
+	if (!lp_idmap_uid(&low_uid, &high_uid)
+	    || !lp_idmap_gid(&low_gid, &high_gid)) {
+		DEBUG(1, ("idmap uid or idmap gid missing\n"));
+		return NT_STATUS_UNSUCCESSFUL;
 	}
 
-	/* Create high water marks for group and user id */
-	if (lp_idmap_uid(&low_uid, &high_uid)) {
-		idmap_tdb_state.low_uid = low_uid;
-		idmap_tdb_state.high_uid = high_uid;
-	}
-
-	if (lp_idmap_gid(&low_gid, &high_gid)) {
-		idmap_tdb_state.low_gid = low_gid;
-		idmap_tdb_state.high_gid = high_gid;
-	}
+	idmap_tdb_state.low_uid = low_uid;
+	idmap_tdb_state.high_uid = high_uid;
+	idmap_tdb_state.low_gid = low_gid;
+	idmap_tdb_state.high_gid = high_gid;
 
 	if (idmap_tdb_state.high_uid <= idmap_tdb_state.low_uid) {
 		DEBUG(1, ("idmap uid range missing or invalid\n"));
 		DEBUGADD(1, ("idmap will be unable to map foreign SIDs\n"));
 		return NT_STATUS_UNSUCCESSFUL;
-	} else {
-		uint32 low_id;
-
-		if (((low_id = tdb_fetch_int32(idmap_alloc_tdb, HWM_USER)) == -1) ||
-		    (low_id < idmap_tdb_state.low_uid)) {
-			if (tdb_store_int32(idmap_alloc_tdb, HWM_USER, idmap_tdb_state.low_uid) == -1) {
-				DEBUG(0, ("Unable to initialise user hwm in idmap database\n"));
-				return NT_STATUS_INTERNAL_DB_ERROR;
-			}
-		}
 	}
 
 	if (idmap_tdb_state.high_gid <= idmap_tdb_state.low_gid) {
 		DEBUG(1, ("idmap gid range missing or invalid\n"));
 		DEBUGADD(1, ("idmap will be unable to map foreign SIDs\n"));
 		return NT_STATUS_UNSUCCESSFUL;
-	} else {
-		uint32 low_id;
+	}
 
-		if (((low_id = tdb_fetch_int32(idmap_alloc_tdb, HWM_GROUP)) == -1) ||
-		    (low_id < idmap_tdb_state.low_gid)) {
-			if (tdb_store_int32(idmap_alloc_tdb, HWM_GROUP, idmap_tdb_state.low_gid) == -1) {
-				DEBUG(0, ("Unable to initialise group hwm in idmap database\n"));
-				return NT_STATUS_INTERNAL_DB_ERROR;
-			}
+	if (((low_id = tdb_fetch_int32(idmap_alloc_tdb, HWM_USER)) == -1) ||
+	    (low_id < idmap_tdb_state.low_uid)) {
+		if (tdb_store_int32(idmap_alloc_tdb, HWM_USER,
+				    idmap_tdb_state.low_uid) == -1) {
+			DEBUG(0, ("Unable to initialise user hwm in idmap "
+				  "database\n"));
+			return NT_STATUS_INTERNAL_DB_ERROR;
+		}
+	}
+
+	if (((low_id = tdb_fetch_int32(idmap_alloc_tdb, HWM_GROUP)) == -1) ||
+	    (low_id < idmap_tdb_state.low_gid)) {
+		if (tdb_store_int32(idmap_alloc_tdb, HWM_GROUP,
+				    idmap_tdb_state.low_gid) == -1) {
+			DEBUG(0, ("Unable to initialise group hwm in idmap "
+				  "database\n"));
+			return NT_STATUS_INTERNAL_DB_ERROR;
 		}
 	}
 
@@ -585,7 +565,7 @@ struct idmap_tdb_context {
  Initialise idmap database. 
 *****************************/
 
-static NTSTATUS idmap_tdb_db_init(struct idmap_domain *dom)
+static NTSTATUS idmap_tdb_db_init(struct idmap_domain *dom, const char *params)
 {
 	NTSTATUS ret;
 	struct idmap_tdb_context *ctx;
@@ -619,7 +599,6 @@ static NTSTATUS idmap_tdb_db_init(struct idmap_domain *dom)
 	}
 
 	dom->private_data = ctx;
-	dom->initialized = True;
 
 	talloc_free(config_option);
 	return NT_STATUS_OK;
@@ -774,14 +753,6 @@ static NTSTATUS idmap_tdb_unixids_to_sids(struct idmap_domain *dom, struct id_ma
 	NTSTATUS ret;
 	int i;
 
-	/* make sure we initialized */
-	if ( ! dom->initialized) {
-		ret = idmap_tdb_db_init(dom);
-		if ( ! NT_STATUS_IS_OK(ret)) {
-			return ret;
-		}
-	}
-
 	ctx = talloc_get_type(dom->private_data, struct idmap_tdb_context);
 
 	for (i = 0; ids[i]; i++) {
@@ -820,14 +791,6 @@ static NTSTATUS idmap_tdb_sids_to_unixids(struct idmap_domain *dom, struct id_ma
 	NTSTATUS ret;
 	int i;
 
-	/* make sure we initialized */
-	if ( ! dom->initialized) {
-		ret = idmap_tdb_db_init(dom);
-		if ( ! NT_STATUS_IS_OK(ret)) {
-			return ret;
-		}
-	}
-
 	ctx = talloc_get_type(dom->private_data, struct idmap_tdb_context);
 
 	for (i = 0; ids[i]; i++) {
@@ -857,10 +820,11 @@ done:
 }
 
 /**********************************
- set a mapping. 
+ set a mapping.
 **********************************/
 
-static NTSTATUS idmap_tdb_set_mapping(struct idmap_domain *dom, const struct id_map *map)
+static NTSTATUS idmap_tdb_set_mapping(struct idmap_domain *dom,
+				      const struct id_map *map)
 {
 	struct idmap_tdb_context *ctx;
 	NTSTATUS ret;
@@ -868,14 +832,6 @@ static NTSTATUS idmap_tdb_set_mapping(struct idmap_domain *dom, const struct id_
 	char *ksidstr, *kidstr;
 	fstring tmp;
 
-	/* make sure we initialized */
-	if ( ! dom->initialized) {
-		ret = idmap_tdb_db_init(dom);
-		if ( ! NT_STATUS_IS_OK(ret)) {
-			return ret;
-		}
-	}
-
 	if (!map || !map->sid) {
 		return NT_STATUS_INVALID_PARAMETER;
 	}
@@ -884,17 +840,19 @@ static NTSTATUS idmap_tdb_set_mapping(struct idmap_domain *dom, const struct id_
 	data.dptr = NULL;
 
 	/* TODO: should we filter a set_mapping using low/high filters ? */
-	
+
 	ctx = talloc_get_type(dom->private_data, struct idmap_tdb_context);
 
 	switch (map->xid.type) {
 
 	case ID_TYPE_UID:
-		kidstr = talloc_asprintf(ctx, "UID %lu", (unsigned long)map->xid.id);
+		kidstr = talloc_asprintf(ctx, "UID %lu",
+					 (unsigned long)map->xid.id);
 		break;
-		
+
 	case ID_TYPE_GID:
-		kidstr = talloc_asprintf(ctx, "GID %lu", (unsigned long)map->xid.id);
+		kidstr = talloc_asprintf(ctx, "GID %lu",
+					 (unsigned long)map->xid.id);
 		break;
 
 	default:
@@ -920,8 +878,13 @@ static NTSTATUS idmap_tdb_set_mapping(struct idmap_domain *dom, const struct id_
 	ksid = string_term_tdb_data(ksidstr);
 
 	/* *DELETE* previous mappings if any.
-	 * This is done both SID and [U|G]ID passed in */
-	
+	 * This is done for both the SID and [U|G]ID passed in */
+
+	/* NOTE: We should lock both the ksid and kid records here, before
+	 * making modifications.  However, because tdb_chainlock() is a
+	 * blocking call we could create an unrecoverable deadlock, so for now
+	 * we only lock the ksid record. */
+
 	/* Lock the record for this SID. */
 	if (tdb_chainlock(ctx->tdb, ksid) != 0) {
 		DEBUG(10,("Failed to lock record %s. Error %s\n",
@@ -931,7 +894,8 @@ static NTSTATUS idmap_tdb_set_mapping(struct idmap_domain *dom, const struct id_
 
 	data = tdb_fetch(ctx->tdb, ksid);
 	if (data.dptr) {
-		DEBUG(10, ("Deleting existing mapping %s <-> %s\n", (const char *)data.dptr, ksidstr ));
+		DEBUG(10, ("Deleting existing mapping %s <-> %s\n",
+		           (const char *)data.dptr, ksidstr ));
 		tdb_delete(ctx->tdb, data);
 		tdb_delete(ctx->tdb, ksid);
 		SAFE_FREE(data.dptr);
@@ -939,20 +903,23 @@ static NTSTATUS idmap_tdb_set_mapping(struct idmap_domain *dom, const struct id_
 
 	data = tdb_fetch(ctx->tdb, kid);
 	if (data.dptr) {
-		DEBUG(10,("Deleting existing mapping %s <-> %s\n", (const char *)data.dptr, kidstr ));
+		DEBUG(10,("Deleting existing mapping %s <-> %s\n",
+			  (const char *)data.dptr, kidstr ));
 		tdb_delete(ctx->tdb, data);
 		tdb_delete(ctx->tdb, kid);
 		SAFE_FREE(data.dptr);
 	}
 
 	if (tdb_store(ctx->tdb, ksid, kid, TDB_INSERT) == -1) {
-		DEBUG(0, ("Error storing SID -> ID: %s\n", tdb_errorstr(ctx->tdb)));
+		DEBUG(0, ("Error storing SID -> ID: %s\n",
+			  tdb_errorstr(ctx->tdb)));
 		tdb_chainunlock(ctx->tdb, ksid);
 		ret = NT_STATUS_UNSUCCESSFUL;
 		goto done;
 	}
 	if (tdb_store(ctx->tdb, kid, ksid, TDB_INSERT) == -1) {
-		DEBUG(0, ("Error stroing ID -> SID: %s\n", tdb_errorstr(ctx->tdb)));
+		DEBUG(0, ("Error storing ID -> SID: %s\n",
+			  tdb_errorstr(ctx->tdb)));
 		/* try to remove the previous stored SID -> ID map */
 		tdb_delete(ctx->tdb, ksid);
 		tdb_chainunlock(ctx->tdb, ksid);
@@ -972,10 +939,11 @@ done:
 }
 
 /**********************************
- remove a mapping. 
+ remove a mapping.
 **********************************/
 
-static NTSTATUS idmap_tdb_remove_mapping(struct idmap_domain *dom, const struct id_map *map)
+static NTSTATUS idmap_tdb_remove_mapping(struct idmap_domain *dom,
+					 const struct id_map *map)
 {
 	struct idmap_tdb_context *ctx;
 	NTSTATUS ret;
@@ -983,14 +951,6 @@ static NTSTATUS idmap_tdb_remove_mapping(struct idmap_domain *dom, const struct
 	char *ksidstr, *kidstr;
 	fstring tmp;
 
-	/* make sure we initialized */
-	if ( ! dom->initialized) {
-		ret = idmap_tdb_db_init(dom);
-		if ( ! NT_STATUS_IS_OK(ret)) {
-			return ret;
-		}
-	}
-
 	if (!map || !map->sid) {
 		return NT_STATUS_INVALID_PARAMETER;
 	}
@@ -999,17 +959,19 @@ static NTSTATUS idmap_tdb_remove_mapping(struct idmap_domain *dom, const struct
 	data.dptr = NULL;
 
 	/* TODO: should we filter a remove_mapping using low/high filters ? */
-	
+
 	ctx = talloc_get_type(dom->private_data, struct idmap_tdb_context);
 
 	switch (map->xid.type) {
 
 	case ID_TYPE_UID:
-		kidstr = talloc_asprintf(ctx, "UID %lu", (unsigned long)map->xid.id);
+		kidstr = talloc_asprintf(ctx, "UID %lu",
+					 (unsigned long)map->xid.id);
 		break;
-		
+
 	case ID_TYPE_GID:
-		kidstr = talloc_asprintf(ctx, "GID %lu", (unsigned long)map->xid.id);
+		kidstr = talloc_asprintf(ctx, "GID %lu",
+					 (unsigned long)map->xid.id);
 		break;
 
 	default:
@@ -1034,6 +996,11 @@ static NTSTATUS idmap_tdb_remove_mapping(struct idmap_domain *dom, const struct
 	ksid = string_term_tdb_data(ksidstr);
 	kid = string_term_tdb_data(kidstr);
 
+	/* NOTE: We should lock both the ksid and kid records here, before
+	 * making modifications.  However, because tdb_chainlock() is a
+	 * blocking call we could create an unrecoverable deadlock, so for now
+	 * we only lock the ksid record. */
+
 	/* Lock the record for this SID. */
 	if (tdb_chainlock(ctx->tdb, ksid) != 0) {
 		DEBUG(10,("Failed to lock record %s. Error %s\n",
@@ -1054,16 +1021,17 @@ static NTSTATUS idmap_tdb_remove_mapping(struct idmap_domain *dom, const struct
 	if ((data.dsize != kid.dsize) ||
 	    (memcmp(data.dptr, kid.dptr, data.dsize) != 0)) {
 		DEBUG(10,("Specified SID does not map to specified ID\n"));
-		DEBUGADD(10,("Actual mapping is %s -> %s\n", ksidstr, (const char *)data.dptr));
+		DEBUGADD(10,("Actual mapping is %s -> %s\n", ksidstr,
+			 (const char *)data.dptr));
 		tdb_chainunlock(ctx->tdb, ksid);
 		ret = NT_STATUS_NONE_MAPPED;
 		goto done;
 	}
-	
+
 	DEBUG(10, ("Removing %s <-> %s map\n", ksidstr, kidstr));
 
 	/* Delete previous mappings. */
-	
+
 	DEBUG(10, ("Deleting existing mapping %s -> %s\n", ksidstr, kidstr ));
 	tdb_delete(ctx->tdb, ksid);
 
@@ -1171,14 +1139,6 @@ static NTSTATUS idmap_tdb_dump_data(struct idmap_domain *dom, struct id_map **ma
 	struct dump_data *data;
 	NTSTATUS ret = NT_STATUS_OK;
 
-	/* make sure we initialized */
-	if ( ! dom->initialized) {
-		ret = idmap_tdb_db_init(dom);
-		if ( ! NT_STATUS_IS_OK(ret)) {
-			return ret;
-		}
-	}
-
 	ctx = talloc_get_type(dom->private_data, struct idmap_tdb_context);
 
 	data = TALLOC_ZERO_P(ctx, struct dump_data);
@@ -1229,6 +1189,8 @@ NTSTATUS idmap_tdb_init(void)
 {
 	NTSTATUS ret;
 
+	DEBUG(10, ("calling idmap_tdb_init\n"));
+
 	/* FIXME: bad hack to actually register also the alloc_tdb module without changining configure.in */
 	ret = idmap_alloc_tdb_init();
 	if (! NT_STATUS_IS_OK(ret)) {
